from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class CommentTypeTag(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
    COMMENTTYPETAGUNKNOWN: _ClassVar[CommentTypeTag]
    COMMENTTYPETAGSTAR: _ClassVar[CommentTypeTag]
COMMENTTYPETAGUNKNOWN: CommentTypeTag
COMMENTTYPETAGSTAR: CommentTypeTag

class Response(_message.Message):
    __slots__ = ["messagesList", "cursor", "fetchInterval", "now", "internalExt", "fetchType", "routeParams", "heartbeatDuration", "needAck", "pushServer", "liveCursor", "historyNoMore"]
    class RouteParamsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    MESSAGESLIST_FIELD_NUMBER: _ClassVar[int]
    CURSOR_FIELD_NUMBER: _ClassVar[int]
    FETCHINTERVAL_FIELD_NUMBER: _ClassVar[int]
    NOW_FIELD_NUMBER: _ClassVar[int]
    INTERNALEXT_FIELD_NUMBER: _ClassVar[int]
    FETCHTYPE_FIELD_NUMBER: _ClassVar[int]
    ROUTEPARAMS_FIELD_NUMBER: _ClassVar[int]
    HEARTBEATDURATION_FIELD_NUMBER: _ClassVar[int]
    NEEDACK_FIELD_NUMBER: _ClassVar[int]
    PUSHSERVER_FIELD_NUMBER: _ClassVar[int]
    LIVECURSOR_FIELD_NUMBER: _ClassVar[int]
    HISTORYNOMORE_FIELD_NUMBER: _ClassVar[int]
    messagesList: _containers.RepeatedCompositeFieldContainer[Message]
    cursor: str
    fetchInterval: int
    now: int
    internalExt: str
    fetchType: int
    routeParams: _containers.ScalarMap[str, str]
    heartbeatDuration: int
    needAck: bool
    pushServer: str
    liveCursor: str
    historyNoMore: bool
    def __init__(self, messagesList: _Optional[_Iterable[_Union[Message, _Mapping]]] = ..., cursor: _Optional[str] = ..., fetchInterval: _Optional[int] = ..., now: _Optional[int] = ..., internalExt: _Optional[str] = ..., fetchType: _Optional[int] = ..., routeParams: _Optional[_Mapping[str, str]] = ..., heartbeatDuration: _Optional[int] = ..., needAck: bool = ..., pushServer: _Optional[str] = ..., liveCursor: _Optional[str] = ..., historyNoMore: bool = ...) -> None: ...

class Message(_message.Message):
    __slots__ = ["method", "payload", "msgId", "msgType", "offset", "needWrdsStore", "wrdsVersion", "wrdsSubKey"]
    METHOD_FIELD_NUMBER: _ClassVar[int]
    PAYLOAD_FIELD_NUMBER: _ClassVar[int]
    MSGID_FIELD_NUMBER: _ClassVar[int]
    MSGTYPE_FIELD_NUMBER: _ClassVar[int]
    OFFSET_FIELD_NUMBER: _ClassVar[int]
    NEEDWRDSSTORE_FIELD_NUMBER: _ClassVar[int]
    WRDSVERSION_FIELD_NUMBER: _ClassVar[int]
    WRDSSUBKEY_FIELD_NUMBER: _ClassVar[int]
    method: str
    payload: bytes
    msgId: int
    msgType: int
    offset: int
    needWrdsStore: bool
    wrdsVersion: int
    wrdsSubKey: str
    def __init__(self, method: _Optional[str] = ..., payload: _Optional[bytes] = ..., msgId: _Optional[int] = ..., msgType: _Optional[int] = ..., offset: _Optional[int] = ..., needWrdsStore: bool = ..., wrdsVersion: _Optional[int] = ..., wrdsSubKey: _Optional[str] = ...) -> None: ...

class ChatMessage(_message.Message):
    __slots__ = ["common", "user", "content", "visibleToSender", "backgroundImage", "fullScreenTextColor", "backgroundImageV2", "publicAreaCommon", "giftImage", "agreeMsgId", "priorityLevel", "landscapeAreaCommon", "eventTime", "sendReview", "fromIntercom", "intercomHideUserCard", "chatBy", "individualChatPriority", "rtfContent"]
    COMMON_FIELD_NUMBER: _ClassVar[int]
    USER_FIELD_NUMBER: _ClassVar[int]
    CONTENT_FIELD_NUMBER: _ClassVar[int]
    VISIBLETOSENDER_FIELD_NUMBER: _ClassVar[int]
    BACKGROUNDIMAGE_FIELD_NUMBER: _ClassVar[int]
    FULLSCREENTEXTCOLOR_FIELD_NUMBER: _ClassVar[int]
    BACKGROUNDIMAGEV2_FIELD_NUMBER: _ClassVar[int]
    PUBLICAREACOMMON_FIELD_NUMBER: _ClassVar[int]
    GIFTIMAGE_FIELD_NUMBER: _ClassVar[int]
    AGREEMSGID_FIELD_NUMBER: _ClassVar[int]
    PRIORITYLEVEL_FIELD_NUMBER: _ClassVar[int]
    LANDSCAPEAREACOMMON_FIELD_NUMBER: _ClassVar[int]
    EVENTTIME_FIELD_NUMBER: _ClassVar[int]
    SENDREVIEW_FIELD_NUMBER: _ClassVar[int]
    FROMINTERCOM_FIELD_NUMBER: _ClassVar[int]
    INTERCOMHIDEUSERCARD_FIELD_NUMBER: _ClassVar[int]
    CHATBY_FIELD_NUMBER: _ClassVar[int]
    INDIVIDUALCHATPRIORITY_FIELD_NUMBER: _ClassVar[int]
    RTFCONTENT_FIELD_NUMBER: _ClassVar[int]
    common: Common
    user: User
    content: str
    visibleToSender: bool
    backgroundImage: Image
    fullScreenTextColor: str
    backgroundImageV2: Image
    publicAreaCommon: PublicAreaCommon
    giftImage: Image
    agreeMsgId: int
    priorityLevel: int
    landscapeAreaCommon: LandscapeAreaCommon
    eventTime: int
    sendReview: bool
    fromIntercom: bool
    intercomHideUserCard: bool
    chatBy: str
    individualChatPriority: int
    rtfContent: Text
    def __init__(self, common: _Optional[_Union[Common, _Mapping]] = ..., user: _Optional[_Union[User, _Mapping]] = ..., content: _Optional[str] = ..., visibleToSender: bool = ..., backgroundImage: _Optional[_Union[Image, _Mapping]] = ..., fullScreenTextColor: _Optional[str] = ..., backgroundImageV2: _Optional[_Union[Image, _Mapping]] = ..., publicAreaCommon: _Optional[_Union[PublicAreaCommon, _Mapping]] = ..., giftImage: _Optional[_Union[Image, _Mapping]] = ..., agreeMsgId: _Optional[int] = ..., priorityLevel: _Optional[int] = ..., landscapeAreaCommon: _Optional[_Union[LandscapeAreaCommon, _Mapping]] = ..., eventTime: _Optional[int] = ..., sendReview: bool = ..., fromIntercom: bool = ..., intercomHideUserCard: bool = ..., chatBy: _Optional[str] = ..., individualChatPriority: _Optional[int] = ..., rtfContent: _Optional[_Union[Text, _Mapping]] = ...) -> None: ...

class LandscapeAreaCommon(_message.Message):
    __slots__ = ["showHead", "showNickname", "showFontColor", "colorValueList", "commentTypeTagsList"]
    SHOWHEAD_FIELD_NUMBER: _ClassVar[int]
    SHOWNICKNAME_FIELD_NUMBER: _ClassVar[int]
    SHOWFONTCOLOR_FIELD_NUMBER: _ClassVar[int]
    COLORVALUELIST_FIELD_NUMBER: _ClassVar[int]
    COMMENTTYPETAGSLIST_FIELD_NUMBER: _ClassVar[int]
    showHead: bool
    showNickname: bool
    showFontColor: bool
    colorValueList: _containers.RepeatedScalarFieldContainer[str]
    commentTypeTagsList: _containers.RepeatedScalarFieldContainer[CommentTypeTag]
    def __init__(self, showHead: bool = ..., showNickname: bool = ..., showFontColor: bool = ..., colorValueList: _Optional[_Iterable[str]] = ..., commentTypeTagsList: _Optional[_Iterable[_Union[CommentTypeTag, str]]] = ...) -> None: ...

class RoomUserSeqMessage(_message.Message):
    __slots__ = ["common", "ranksList", "total", "popStr", "seatsList", "popularity", "totalUser", "totalUserStr", "totalStr", "onlineUserForAnchor", "totalPvForAnchor", "upRightStatsStr", "upRightStatsStrComplete"]
    COMMON_FIELD_NUMBER: _ClassVar[int]
    RANKSLIST_FIELD_NUMBER: _ClassVar[int]
    TOTAL_FIELD_NUMBER: _ClassVar[int]
    POPSTR_FIELD_NUMBER: _ClassVar[int]
    SEATSLIST_FIELD_NUMBER: _ClassVar[int]
    POPULARITY_FIELD_NUMBER: _ClassVar[int]
    TOTALUSER_FIELD_NUMBER: _ClassVar[int]
    TOTALUSERSTR_FIELD_NUMBER: _ClassVar[int]
    TOTALSTR_FIELD_NUMBER: _ClassVar[int]
    ONLINEUSERFORANCHOR_FIELD_NUMBER: _ClassVar[int]
    TOTALPVFORANCHOR_FIELD_NUMBER: _ClassVar[int]
    UPRIGHTSTATSSTR_FIELD_NUMBER: _ClassVar[int]
    UPRIGHTSTATSSTRCOMPLETE_FIELD_NUMBER: _ClassVar[int]
    common: Common
    ranksList: _containers.RepeatedCompositeFieldContainer[RoomUserSeqMessageContributor]
    total: int
    popStr: str
    seatsList: _containers.RepeatedCompositeFieldContainer[RoomUserSeqMessageContributor]
    popularity: int
    totalUser: int
    totalUserStr: str
    totalStr: str
    onlineUserForAnchor: str
    totalPvForAnchor: str
    upRightStatsStr: str
    upRightStatsStrComplete: str
    def __init__(self, common: _Optional[_Union[Common, _Mapping]] = ..., ranksList: _Optional[_Iterable[_Union[RoomUserSeqMessageContributor, _Mapping]]] = ..., total: _Optional[int] = ..., popStr: _Optional[str] = ..., seatsList: _Optional[_Iterable[_Union[RoomUserSeqMessageContributor, _Mapping]]] = ..., popularity: _Optional[int] = ..., totalUser: _Optional[int] = ..., totalUserStr: _Optional[str] = ..., totalStr: _Optional[str] = ..., onlineUserForAnchor: _Optional[str] = ..., totalPvForAnchor: _Optional[str] = ..., upRightStatsStr: _Optional[str] = ..., upRightStatsStrComplete: _Optional[str] = ...) -> None: ...

class CommonTextMessage(_message.Message):
    __slots__ = ["common", "user", "scene"]
    COMMON_FIELD_NUMBER: _ClassVar[int]
    USER_FIELD_NUMBER: _ClassVar[int]
    SCENE_FIELD_NUMBER: _ClassVar[int]
    common: Common
    user: User
    scene: str
    def __init__(self, common: _Optional[_Union[Common, _Mapping]] = ..., user: _Optional[_Union[User, _Mapping]] = ..., scene: _Optional[str] = ...) -> None: ...

class UpdateFanTicketMessage(_message.Message):
    __slots__ = ["common", "roomFanTicketCountText", "roomFanTicketCount", "forceUpdate"]
    COMMON_FIELD_NUMBER: _ClassVar[int]
    ROOMFANTICKETCOUNTTEXT_FIELD_NUMBER: _ClassVar[int]
    ROOMFANTICKETCOUNT_FIELD_NUMBER: _ClassVar[int]
    FORCEUPDATE_FIELD_NUMBER: _ClassVar[int]
    common: Common
    roomFanTicketCountText: str
    roomFanTicketCount: int
    forceUpdate: bool
    def __init__(self, common: _Optional[_Union[Common, _Mapping]] = ..., roomFanTicketCountText: _Optional[str] = ..., roomFanTicketCount: _Optional[int] = ..., forceUpdate: bool = ...) -> None: ...

class RoomUserSeqMessageContributor(_message.Message):
    __slots__ = ["score", "user", "rank", "delta", "isHidden", "scoreDescription", "exactlyScore"]
    SCORE_FIELD_NUMBER: _ClassVar[int]
    USER_FIELD_NUMBER: _ClassVar[int]
    RANK_FIELD_NUMBER: _ClassVar[int]
    DELTA_FIELD_NUMBER: _ClassVar[int]
    ISHIDDEN_FIELD_NUMBER: _ClassVar[int]
    SCOREDESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    EXACTLYSCORE_FIELD_NUMBER: _ClassVar[int]
    score: int
    user: User
    rank: int
    delta: int
    isHidden: bool
    scoreDescription: str
    exactlyScore: str
    def __init__(self, score: _Optional[int] = ..., user: _Optional[_Union[User, _Mapping]] = ..., rank: _Optional[int] = ..., delta: _Optional[int] = ..., isHidden: bool = ..., scoreDescription: _Optional[str] = ..., exactlyScore: _Optional[str] = ...) -> None: ...

class GiftMessage(_message.Message):
    __slots__ = ["common", "giftId", "fanTicketCount", "groupCount", "repeatCount", "comboCount", "user", "toUser", "repeatEnd", "textEffect", "groupId", "incomeTaskgifts", "roomFanTicketCount", "priority", "gift", "logId", "sendType", "publicAreaCommon", "trayDisplayText", "bannedDisplayEffects", "displayForSelf", "interactGiftInfo", "diyItemInfo", "minAssetSetList", "totalCount", "clientGiftSource", "toUserIdsList", "sendTime", "forceDisplayEffects", "traceId", "effectDisplayTs"]
    COMMON_FIELD_NUMBER: _ClassVar[int]
    GIFTID_FIELD_NUMBER: _ClassVar[int]
    FANTICKETCOUNT_FIELD_NUMBER: _ClassVar[int]
    GROUPCOUNT_FIELD_NUMBER: _ClassVar[int]
    REPEATCOUNT_FIELD_NUMBER: _ClassVar[int]
    COMBOCOUNT_FIELD_NUMBER: _ClassVar[int]
    USER_FIELD_NUMBER: _ClassVar[int]
    TOUSER_FIELD_NUMBER: _ClassVar[int]
    REPEATEND_FIELD_NUMBER: _ClassVar[int]
    TEXTEFFECT_FIELD_NUMBER: _ClassVar[int]
    GROUPID_FIELD_NUMBER: _ClassVar[int]
    INCOMETASKGIFTS_FIELD_NUMBER: _ClassVar[int]
    ROOMFANTICKETCOUNT_FIELD_NUMBER: _ClassVar[int]
    PRIORITY_FIELD_NUMBER: _ClassVar[int]
    GIFT_FIELD_NUMBER: _ClassVar[int]
    LOGID_FIELD_NUMBER: _ClassVar[int]
    SENDTYPE_FIELD_NUMBER: _ClassVar[int]
    PUBLICAREACOMMON_FIELD_NUMBER: _ClassVar[int]
    TRAYDISPLAYTEXT_FIELD_NUMBER: _ClassVar[int]
    BANNEDDISPLAYEFFECTS_FIELD_NUMBER: _ClassVar[int]
    DISPLAYFORSELF_FIELD_NUMBER: _ClassVar[int]
    INTERACTGIFTINFO_FIELD_NUMBER: _ClassVar[int]
    DIYITEMINFO_FIELD_NUMBER: _ClassVar[int]
    MINASSETSETLIST_FIELD_NUMBER: _ClassVar[int]
    TOTALCOUNT_FIELD_NUMBER: _ClassVar[int]
    CLIENTGIFTSOURCE_FIELD_NUMBER: _ClassVar[int]
    TOUSERIDSLIST_FIELD_NUMBER: _ClassVar[int]
    SENDTIME_FIELD_NUMBER: _ClassVar[int]
    FORCEDISPLAYEFFECTS_FIELD_NUMBER: _ClassVar[int]
    TRACEID_FIELD_NUMBER: _ClassVar[int]
    EFFECTDISPLAYTS_FIELD_NUMBER: _ClassVar[int]
    common: Common
    giftId: int
    fanTicketCount: int
    groupCount: int
    repeatCount: int
    comboCount: int
    user: User
    toUser: User
    repeatEnd: int
    textEffect: TextEffect
    groupId: int
    incomeTaskgifts: int
    roomFanTicketCount: int
    priority: GiftIMPriority
    gift: GiftStruct
    logId: str
    sendType: int
    publicAreaCommon: PublicAreaCommon
    trayDisplayText: Text
    bannedDisplayEffects: int
    displayForSelf: bool
    interactGiftInfo: str
    diyItemInfo: str
    minAssetSetList: _containers.RepeatedScalarFieldContainer[int]
    totalCount: int
    clientGiftSource: int
    toUserIdsList: _containers.RepeatedScalarFieldContainer[int]
    sendTime: int
    forceDisplayEffects: int
    traceId: str
    effectDisplayTs: int
    def __init__(self, common: _Optional[_Union[Common, _Mapping]] = ..., giftId: _Optional[int] = ..., fanTicketCount: _Optional[int] = ..., groupCount: _Optional[int] = ..., repeatCount: _Optional[int] = ..., comboCount: _Optional[int] = ..., user: _Optional[_Union[User, _Mapping]] = ..., toUser: _Optional[_Union[User, _Mapping]] = ..., repeatEnd: _Optional[int] = ..., textEffect: _Optional[_Union[TextEffect, _Mapping]] = ..., groupId: _Optional[int] = ..., incomeTaskgifts: _Optional[int] = ..., roomFanTicketCount: _Optional[int] = ..., priority: _Optional[_Union[GiftIMPriority, _Mapping]] = ..., gift: _Optional[_Union[GiftStruct, _Mapping]] = ..., logId: _Optional[str] = ..., sendType: _Optional[int] = ..., publicAreaCommon: _Optional[_Union[PublicAreaCommon, _Mapping]] = ..., trayDisplayText: _Optional[_Union[Text, _Mapping]] = ..., bannedDisplayEffects: _Optional[int] = ..., displayForSelf: bool = ..., interactGiftInfo: _Optional[str] = ..., diyItemInfo: _Optional[str] = ..., minAssetSetList: _Optional[_Iterable[int]] = ..., totalCount: _Optional[int] = ..., clientGiftSource: _Optional[int] = ..., toUserIdsList: _Optional[_Iterable[int]] = ..., sendTime: _Optional[int] = ..., forceDisplayEffects: _Optional[int] = ..., traceId: _Optional[str] = ..., effectDisplayTs: _Optional[int] = ...) -> None: ...

class GiftStruct(_message.Message):
    __slots__ = ["image", "describe", "notify", "duration", "id", "forLinkmic", "doodle", "forFansclub", "combo", "type", "diamondCount", "isDisplayedOnPanel", "primaryEffectId", "giftLabelIcon", "name", "region", "manual", "forCustom", "icon", "actionType"]
    IMAGE_FIELD_NUMBER: _ClassVar[int]
    DESCRIBE_FIELD_NUMBER: _ClassVar[int]
    NOTIFY_FIELD_NUMBER: _ClassVar[int]
    DURATION_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    FORLINKMIC_FIELD_NUMBER: _ClassVar[int]
    DOODLE_FIELD_NUMBER: _ClassVar[int]
    FORFANSCLUB_FIELD_NUMBER: _ClassVar[int]
    COMBO_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    DIAMONDCOUNT_FIELD_NUMBER: _ClassVar[int]
    ISDISPLAYEDONPANEL_FIELD_NUMBER: _ClassVar[int]
    PRIMARYEFFECTID_FIELD_NUMBER: _ClassVar[int]
    GIFTLABELICON_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    REGION_FIELD_NUMBER: _ClassVar[int]
    MANUAL_FIELD_NUMBER: _ClassVar[int]
    FORCUSTOM_FIELD_NUMBER: _ClassVar[int]
    ICON_FIELD_NUMBER: _ClassVar[int]
    ACTIONTYPE_FIELD_NUMBER: _ClassVar[int]
    image: Image
    describe: str
    notify: bool
    duration: int
    id: int
    forLinkmic: bool
    doodle: bool
    forFansclub: bool
    combo: bool
    type: int
    diamondCount: int
    isDisplayedOnPanel: bool
    primaryEffectId: int
    giftLabelIcon: Image
    name: str
    region: str
    manual: str
    forCustom: bool
    icon: Image
    actionType: int
    def __init__(self, image: _Optional[_Union[Image, _Mapping]] = ..., describe: _Optional[str] = ..., notify: bool = ..., duration: _Optional[int] = ..., id: _Optional[int] = ..., forLinkmic: bool = ..., doodle: bool = ..., forFansclub: bool = ..., combo: bool = ..., type: _Optional[int] = ..., diamondCount: _Optional[int] = ..., isDisplayedOnPanel: bool = ..., primaryEffectId: _Optional[int] = ..., giftLabelIcon: _Optional[_Union[Image, _Mapping]] = ..., name: _Optional[str] = ..., region: _Optional[str] = ..., manual: _Optional[str] = ..., forCustom: bool = ..., icon: _Optional[_Union[Image, _Mapping]] = ..., actionType: _Optional[int] = ...) -> None: ...

class GiftIMPriority(_message.Message):
    __slots__ = ["queueSizesList", "selfQueuePriority", "priority"]
    QUEUESIZESLIST_FIELD_NUMBER: _ClassVar[int]
    SELFQUEUEPRIORITY_FIELD_NUMBER: _ClassVar[int]
    PRIORITY_FIELD_NUMBER: _ClassVar[int]
    queueSizesList: _containers.RepeatedScalarFieldContainer[int]
    selfQueuePriority: int
    priority: int
    def __init__(self, queueSizesList: _Optional[_Iterable[int]] = ..., selfQueuePriority: _Optional[int] = ..., priority: _Optional[int] = ...) -> None: ...

class TextEffect(_message.Message):
    __slots__ = ["portrait", "landscape"]
    PORTRAIT_FIELD_NUMBER: _ClassVar[int]
    LANDSCAPE_FIELD_NUMBER: _ClassVar[int]
    portrait: TextEffectDetail
    landscape: TextEffectDetail
    def __init__(self, portrait: _Optional[_Union[TextEffectDetail, _Mapping]] = ..., landscape: _Optional[_Union[TextEffectDetail, _Mapping]] = ...) -> None: ...

class TextEffectDetail(_message.Message):
    __slots__ = ["text", "textFontSize", "background", "start", "duration", "x", "y", "width", "height", "shadowDx", "shadowDy", "shadowRadius", "shadowColor", "strokeColor", "strokeWidth"]
    TEXT_FIELD_NUMBER: _ClassVar[int]
    TEXTFONTSIZE_FIELD_NUMBER: _ClassVar[int]
    BACKGROUND_FIELD_NUMBER: _ClassVar[int]
    START_FIELD_NUMBER: _ClassVar[int]
    DURATION_FIELD_NUMBER: _ClassVar[int]
    X_FIELD_NUMBER: _ClassVar[int]
    Y_FIELD_NUMBER: _ClassVar[int]
    WIDTH_FIELD_NUMBER: _ClassVar[int]
    HEIGHT_FIELD_NUMBER: _ClassVar[int]
    SHADOWDX_FIELD_NUMBER: _ClassVar[int]
    SHADOWDY_FIELD_NUMBER: _ClassVar[int]
    SHADOWRADIUS_FIELD_NUMBER: _ClassVar[int]
    SHADOWCOLOR_FIELD_NUMBER: _ClassVar[int]
    STROKECOLOR_FIELD_NUMBER: _ClassVar[int]
    STROKEWIDTH_FIELD_NUMBER: _ClassVar[int]
    text: Text
    textFontSize: int
    background: Image
    start: int
    duration: int
    x: int
    y: int
    width: int
    height: int
    shadowDx: int
    shadowDy: int
    shadowRadius: int
    shadowColor: str
    strokeColor: str
    strokeWidth: int
    def __init__(self, text: _Optional[_Union[Text, _Mapping]] = ..., textFontSize: _Optional[int] = ..., background: _Optional[_Union[Image, _Mapping]] = ..., start: _Optional[int] = ..., duration: _Optional[int] = ..., x: _Optional[int] = ..., y: _Optional[int] = ..., width: _Optional[int] = ..., height: _Optional[int] = ..., shadowDx: _Optional[int] = ..., shadowDy: _Optional[int] = ..., shadowRadius: _Optional[int] = ..., shadowColor: _Optional[str] = ..., strokeColor: _Optional[str] = ..., strokeWidth: _Optional[int] = ...) -> None: ...

class MemberMessage(_message.Message):
    __slots__ = ["common", "user", "memberCount", "operator", "isSetToAdmin", "isTopUser", "rankScore", "topUserNo", "enterType", "action", "actionDescription", "userId", "effectConfig", "popStr", "enterEffectConfig", "backgroundImage", "backgroundImageV2", "anchorDisplayText", "publicAreaCommon", "userEnterTipType", "anchorEnterTipType"]
    COMMON_FIELD_NUMBER: _ClassVar[int]
    USER_FIELD_NUMBER: _ClassVar[int]
    MEMBERCOUNT_FIELD_NUMBER: _ClassVar[int]
    OPERATOR_FIELD_NUMBER: _ClassVar[int]
    ISSETTOADMIN_FIELD_NUMBER: _ClassVar[int]
    ISTOPUSER_FIELD_NUMBER: _ClassVar[int]
    RANKSCORE_FIELD_NUMBER: _ClassVar[int]
    TOPUSERNO_FIELD_NUMBER: _ClassVar[int]
    ENTERTYPE_FIELD_NUMBER: _ClassVar[int]
    ACTION_FIELD_NUMBER: _ClassVar[int]
    ACTIONDESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    USERID_FIELD_NUMBER: _ClassVar[int]
    EFFECTCONFIG_FIELD_NUMBER: _ClassVar[int]
    POPSTR_FIELD_NUMBER: _ClassVar[int]
    ENTEREFFECTCONFIG_FIELD_NUMBER: _ClassVar[int]
    BACKGROUNDIMAGE_FIELD_NUMBER: _ClassVar[int]
    BACKGROUNDIMAGEV2_FIELD_NUMBER: _ClassVar[int]
    ANCHORDISPLAYTEXT_FIELD_NUMBER: _ClassVar[int]
    PUBLICAREACOMMON_FIELD_NUMBER: _ClassVar[int]
    USERENTERTIPTYPE_FIELD_NUMBER: _ClassVar[int]
    ANCHORENTERTIPTYPE_FIELD_NUMBER: _ClassVar[int]
    common: Common
    user: User
    memberCount: int
    operator: User
    isSetToAdmin: bool
    isTopUser: bool
    rankScore: int
    topUserNo: int
    enterType: int
    action: int
    actionDescription: str
    userId: int
    effectConfig: EffectConfig
    popStr: str
    enterEffectConfig: EffectConfig
    backgroundImage: Image
    backgroundImageV2: Image
    anchorDisplayText: Text
    publicAreaCommon: PublicAreaCommon
    userEnterTipType: int
    anchorEnterTipType: int
    def __init__(self, common: _Optional[_Union[Common, _Mapping]] = ..., user: _Optional[_Union[User, _Mapping]] = ..., memberCount: _Optional[int] = ..., operator: _Optional[_Union[User, _Mapping]] = ..., isSetToAdmin: bool = ..., isTopUser: bool = ..., rankScore: _Optional[int] = ..., topUserNo: _Optional[int] = ..., enterType: _Optional[int] = ..., action: _Optional[int] = ..., actionDescription: _Optional[str] = ..., userId: _Optional[int] = ..., effectConfig: _Optional[_Union[EffectConfig, _Mapping]] = ..., popStr: _Optional[str] = ..., enterEffectConfig: _Optional[_Union[EffectConfig, _Mapping]] = ..., backgroundImage: _Optional[_Union[Image, _Mapping]] = ..., backgroundImageV2: _Optional[_Union[Image, _Mapping]] = ..., anchorDisplayText: _Optional[_Union[Text, _Mapping]] = ..., publicAreaCommon: _Optional[_Union[PublicAreaCommon, _Mapping]] = ..., userEnterTipType: _Optional[int] = ..., anchorEnterTipType: _Optional[int] = ...) -> None: ...

class PublicAreaCommon(_message.Message):
    __slots__ = ["userLabel", "userConsumeInRoom", "userSendGiftCntInRoom"]
    USERLABEL_FIELD_NUMBER: _ClassVar[int]
    USERCONSUMEINROOM_FIELD_NUMBER: _ClassVar[int]
    USERSENDGIFTCNTINROOM_FIELD_NUMBER: _ClassVar[int]
    userLabel: Image
    userConsumeInRoom: int
    userSendGiftCntInRoom: int
    def __init__(self, userLabel: _Optional[_Union[Image, _Mapping]] = ..., userConsumeInRoom: _Optional[int] = ..., userSendGiftCntInRoom: _Optional[int] = ...) -> None: ...

class EffectConfig(_message.Message):
    __slots__ = ["type", "icon", "avatarPos", "text", "textIcon", "stayTime", "animAssetId", "badge", "flexSettingArrayList", "textIconOverlay", "animatedBadge", "hasSweepLight", "textFlexSettingArrayList", "centerAnimAssetId", "dynamicImage", "extraMap", "mp4AnimAssetId", "priority", "maxWaitTime", "dressId", "alignment", "alignmentOffset"]
    class ExtraMapEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    TYPE_FIELD_NUMBER: _ClassVar[int]
    ICON_FIELD_NUMBER: _ClassVar[int]
    AVATARPOS_FIELD_NUMBER: _ClassVar[int]
    TEXT_FIELD_NUMBER: _ClassVar[int]
    TEXTICON_FIELD_NUMBER: _ClassVar[int]
    STAYTIME_FIELD_NUMBER: _ClassVar[int]
    ANIMASSETID_FIELD_NUMBER: _ClassVar[int]
    BADGE_FIELD_NUMBER: _ClassVar[int]
    FLEXSETTINGARRAYLIST_FIELD_NUMBER: _ClassVar[int]
    TEXTICONOVERLAY_FIELD_NUMBER: _ClassVar[int]
    ANIMATEDBADGE_FIELD_NUMBER: _ClassVar[int]
    HASSWEEPLIGHT_FIELD_NUMBER: _ClassVar[int]
    TEXTFLEXSETTINGARRAYLIST_FIELD_NUMBER: _ClassVar[int]
    CENTERANIMASSETID_FIELD_NUMBER: _ClassVar[int]
    DYNAMICIMAGE_FIELD_NUMBER: _ClassVar[int]
    EXTRAMAP_FIELD_NUMBER: _ClassVar[int]
    MP4ANIMASSETID_FIELD_NUMBER: _ClassVar[int]
    PRIORITY_FIELD_NUMBER: _ClassVar[int]
    MAXWAITTIME_FIELD_NUMBER: _ClassVar[int]
    DRESSID_FIELD_NUMBER: _ClassVar[int]
    ALIGNMENT_FIELD_NUMBER: _ClassVar[int]
    ALIGNMENTOFFSET_FIELD_NUMBER: _ClassVar[int]
    type: int
    icon: Image
    avatarPos: int
    text: Text
    textIcon: Image
    stayTime: int
    animAssetId: int
    badge: Image
    flexSettingArrayList: _containers.RepeatedScalarFieldContainer[int]
    textIconOverlay: Image
    animatedBadge: Image
    hasSweepLight: bool
    textFlexSettingArrayList: _containers.RepeatedScalarFieldContainer[int]
    centerAnimAssetId: int
    dynamicImage: Image
    extraMap: _containers.ScalarMap[str, str]
    mp4AnimAssetId: int
    priority: int
    maxWaitTime: int
    dressId: str
    alignment: int
    alignmentOffset: int
    def __init__(self, type: _Optional[int] = ..., icon: _Optional[_Union[Image, _Mapping]] = ..., avatarPos: _Optional[int] = ..., text: _Optional[_Union[Text, _Mapping]] = ..., textIcon: _Optional[_Union[Image, _Mapping]] = ..., stayTime: _Optional[int] = ..., animAssetId: _Optional[int] = ..., badge: _Optional[_Union[Image, _Mapping]] = ..., flexSettingArrayList: _Optional[_Iterable[int]] = ..., textIconOverlay: _Optional[_Union[Image, _Mapping]] = ..., animatedBadge: _Optional[_Union[Image, _Mapping]] = ..., hasSweepLight: bool = ..., textFlexSettingArrayList: _Optional[_Iterable[int]] = ..., centerAnimAssetId: _Optional[int] = ..., dynamicImage: _Optional[_Union[Image, _Mapping]] = ..., extraMap: _Optional[_Mapping[str, str]] = ..., mp4AnimAssetId: _Optional[int] = ..., priority: _Optional[int] = ..., maxWaitTime: _Optional[int] = ..., dressId: _Optional[str] = ..., alignment: _Optional[int] = ..., alignmentOffset: _Optional[int] = ...) -> None: ...

class Text(_message.Message):
    __slots__ = ["key", "defaultPatter", "defaultFormat", "piecesList"]
    KEY_FIELD_NUMBER: _ClassVar[int]
    DEFAULTPATTER_FIELD_NUMBER: _ClassVar[int]
    DEFAULTFORMAT_FIELD_NUMBER: _ClassVar[int]
    PIECESLIST_FIELD_NUMBER: _ClassVar[int]
    key: str
    defaultPatter: str
    defaultFormat: TextFormat
    piecesList: _containers.RepeatedCompositeFieldContainer[TextPiece]
    def __init__(self, key: _Optional[str] = ..., defaultPatter: _Optional[str] = ..., defaultFormat: _Optional[_Union[TextFormat, _Mapping]] = ..., piecesList: _Optional[_Iterable[_Union[TextPiece, _Mapping]]] = ...) -> None: ...

class TextPiece(_message.Message):
    __slots__ = ["type", "format", "stringValue", "userValue", "giftValue", "heartValue", "patternRefValue", "imageValue"]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    FORMAT_FIELD_NUMBER: _ClassVar[int]
    STRINGVALUE_FIELD_NUMBER: _ClassVar[int]
    USERVALUE_FIELD_NUMBER: _ClassVar[int]
    GIFTVALUE_FIELD_NUMBER: _ClassVar[int]
    HEARTVALUE_FIELD_NUMBER: _ClassVar[int]
    PATTERNREFVALUE_FIELD_NUMBER: _ClassVar[int]
    IMAGEVALUE_FIELD_NUMBER: _ClassVar[int]
    type: bool
    format: TextFormat
    stringValue: str
    userValue: TextPieceUser
    giftValue: TextPieceGift
    heartValue: TextPieceHeart
    patternRefValue: TextPiecePatternRef
    imageValue: TextPieceImage
    def __init__(self, type: bool = ..., format: _Optional[_Union[TextFormat, _Mapping]] = ..., stringValue: _Optional[str] = ..., userValue: _Optional[_Union[TextPieceUser, _Mapping]] = ..., giftValue: _Optional[_Union[TextPieceGift, _Mapping]] = ..., heartValue: _Optional[_Union[TextPieceHeart, _Mapping]] = ..., patternRefValue: _Optional[_Union[TextPiecePatternRef, _Mapping]] = ..., imageValue: _Optional[_Union[TextPieceImage, _Mapping]] = ...) -> None: ...

class TextPieceImage(_message.Message):
    __slots__ = ["image", "scalingRate"]
    IMAGE_FIELD_NUMBER: _ClassVar[int]
    SCALINGRATE_FIELD_NUMBER: _ClassVar[int]
    image: Image
    scalingRate: float
    def __init__(self, image: _Optional[_Union[Image, _Mapping]] = ..., scalingRate: _Optional[float] = ...) -> None: ...

class TextPiecePatternRef(_message.Message):
    __slots__ = ["key", "defaultPattern"]
    KEY_FIELD_NUMBER: _ClassVar[int]
    DEFAULTPATTERN_FIELD_NUMBER: _ClassVar[int]
    key: str
    defaultPattern: str
    def __init__(self, key: _Optional[str] = ..., defaultPattern: _Optional[str] = ...) -> None: ...

class TextPieceHeart(_message.Message):
    __slots__ = ["color"]
    COLOR_FIELD_NUMBER: _ClassVar[int]
    color: str
    def __init__(self, color: _Optional[str] = ...) -> None: ...

class TextPieceGift(_message.Message):
    __slots__ = ["giftId", "nameRef"]
    GIFTID_FIELD_NUMBER: _ClassVar[int]
    NAMEREF_FIELD_NUMBER: _ClassVar[int]
    giftId: int
    nameRef: PatternRef
    def __init__(self, giftId: _Optional[int] = ..., nameRef: _Optional[_Union[PatternRef, _Mapping]] = ...) -> None: ...

class PatternRef(_message.Message):
    __slots__ = ["key", "defaultPattern"]
    KEY_FIELD_NUMBER: _ClassVar[int]
    DEFAULTPATTERN_FIELD_NUMBER: _ClassVar[int]
    key: str
    defaultPattern: str
    def __init__(self, key: _Optional[str] = ..., defaultPattern: _Optional[str] = ...) -> None: ...

class TextPieceUser(_message.Message):
    __slots__ = ["user", "withColon"]
    USER_FIELD_NUMBER: _ClassVar[int]
    WITHCOLON_FIELD_NUMBER: _ClassVar[int]
    user: User
    withColon: bool
    def __init__(self, user: _Optional[_Union[User, _Mapping]] = ..., withColon: bool = ...) -> None: ...

class TextFormat(_message.Message):
    __slots__ = ["color", "bold", "italic", "weight", "italicAngle", "fontSize", "useHeighLightColor", "useRemoteClor"]
    COLOR_FIELD_NUMBER: _ClassVar[int]
    BOLD_FIELD_NUMBER: _ClassVar[int]
    ITALIC_FIELD_NUMBER: _ClassVar[int]
    WEIGHT_FIELD_NUMBER: _ClassVar[int]
    ITALICANGLE_FIELD_NUMBER: _ClassVar[int]
    FONTSIZE_FIELD_NUMBER: _ClassVar[int]
    USEHEIGHLIGHTCOLOR_FIELD_NUMBER: _ClassVar[int]
    USEREMOTECLOR_FIELD_NUMBER: _ClassVar[int]
    color: str
    bold: bool
    italic: bool
    weight: int
    italicAngle: int
    fontSize: int
    useHeighLightColor: bool
    useRemoteClor: bool
    def __init__(self, color: _Optional[str] = ..., bold: bool = ..., italic: bool = ..., weight: _Optional[int] = ..., italicAngle: _Optional[int] = ..., fontSize: _Optional[int] = ..., useHeighLightColor: bool = ..., useRemoteClor: bool = ...) -> None: ...

class LikeMessage(_message.Message):
    __slots__ = ["common", "count", "total", "color", "user", "icon", "doubleLikeDetail", "displayControlInfo", "linkmicGuestUid", "scene", "picoDisplayInfo"]
    COMMON_FIELD_NUMBER: _ClassVar[int]
    COUNT_FIELD_NUMBER: _ClassVar[int]
    TOTAL_FIELD_NUMBER: _ClassVar[int]
    COLOR_FIELD_NUMBER: _ClassVar[int]
    USER_FIELD_NUMBER: _ClassVar[int]
    ICON_FIELD_NUMBER: _ClassVar[int]
    DOUBLELIKEDETAIL_FIELD_NUMBER: _ClassVar[int]
    DISPLAYCONTROLINFO_FIELD_NUMBER: _ClassVar[int]
    LINKMICGUESTUID_FIELD_NUMBER: _ClassVar[int]
    SCENE_FIELD_NUMBER: _ClassVar[int]
    PICODISPLAYINFO_FIELD_NUMBER: _ClassVar[int]
    common: Common
    count: int
    total: int
    color: int
    user: User
    icon: str
    doubleLikeDetail: DoubleLikeDetail
    displayControlInfo: DisplayControlInfo
    linkmicGuestUid: int
    scene: str
    picoDisplayInfo: PicoDisplayInfo
    def __init__(self, common: _Optional[_Union[Common, _Mapping]] = ..., count: _Optional[int] = ..., total: _Optional[int] = ..., color: _Optional[int] = ..., user: _Optional[_Union[User, _Mapping]] = ..., icon: _Optional[str] = ..., doubleLikeDetail: _Optional[_Union[DoubleLikeDetail, _Mapping]] = ..., displayControlInfo: _Optional[_Union[DisplayControlInfo, _Mapping]] = ..., linkmicGuestUid: _Optional[int] = ..., scene: _Optional[str] = ..., picoDisplayInfo: _Optional[_Union[PicoDisplayInfo, _Mapping]] = ...) -> None: ...

class SocialMessage(_message.Message):
    __slots__ = ["common", "user", "shareType", "action", "shareTarget", "followCount", "publicAreaCommon"]
    COMMON_FIELD_NUMBER: _ClassVar[int]
    USER_FIELD_NUMBER: _ClassVar[int]
    SHARETYPE_FIELD_NUMBER: _ClassVar[int]
    ACTION_FIELD_NUMBER: _ClassVar[int]
    SHARETARGET_FIELD_NUMBER: _ClassVar[int]
    FOLLOWCOUNT_FIELD_NUMBER: _ClassVar[int]
    PUBLICAREACOMMON_FIELD_NUMBER: _ClassVar[int]
    common: Common
    user: User
    shareType: int
    action: int
    shareTarget: str
    followCount: int
    publicAreaCommon: PublicAreaCommon
    def __init__(self, common: _Optional[_Union[Common, _Mapping]] = ..., user: _Optional[_Union[User, _Mapping]] = ..., shareType: _Optional[int] = ..., action: _Optional[int] = ..., shareTarget: _Optional[str] = ..., followCount: _Optional[int] = ..., publicAreaCommon: _Optional[_Union[PublicAreaCommon, _Mapping]] = ...) -> None: ...

class PicoDisplayInfo(_message.Message):
    __slots__ = ["comboSumCount", "emoji", "emojiIcon", "emojiText"]
    COMBOSUMCOUNT_FIELD_NUMBER: _ClassVar[int]
    EMOJI_FIELD_NUMBER: _ClassVar[int]
    EMOJIICON_FIELD_NUMBER: _ClassVar[int]
    EMOJITEXT_FIELD_NUMBER: _ClassVar[int]
    comboSumCount: int
    emoji: str
    emojiIcon: Image
    emojiText: str
    def __init__(self, comboSumCount: _Optional[int] = ..., emoji: _Optional[str] = ..., emojiIcon: _Optional[_Union[Image, _Mapping]] = ..., emojiText: _Optional[str] = ...) -> None: ...

class DoubleLikeDetail(_message.Message):
    __slots__ = ["doubleFlag", "seqId", "renewalsNum", "triggersNum"]
    DOUBLEFLAG_FIELD_NUMBER: _ClassVar[int]
    SEQID_FIELD_NUMBER: _ClassVar[int]
    RENEWALSNUM_FIELD_NUMBER: _ClassVar[int]
    TRIGGERSNUM_FIELD_NUMBER: _ClassVar[int]
    doubleFlag: bool
    seqId: int
    renewalsNum: int
    triggersNum: int
    def __init__(self, doubleFlag: bool = ..., seqId: _Optional[int] = ..., renewalsNum: _Optional[int] = ..., triggersNum: _Optional[int] = ...) -> None: ...

class DisplayControlInfo(_message.Message):
    __slots__ = ["showText", "showIcons"]
    SHOWTEXT_FIELD_NUMBER: _ClassVar[int]
    SHOWICONS_FIELD_NUMBER: _ClassVar[int]
    showText: bool
    showIcons: bool
    def __init__(self, showText: bool = ..., showIcons: bool = ...) -> None: ...

class EpisodeChatMessage(_message.Message):
    __slots__ = ["common", "user", "content", "visibleToSende", "giftImage", "agreeMsgId", "colorValueList"]
    COMMON_FIELD_NUMBER: _ClassVar[int]
    USER_FIELD_NUMBER: _ClassVar[int]
    CONTENT_FIELD_NUMBER: _ClassVar[int]
    VISIBLETOSENDE_FIELD_NUMBER: _ClassVar[int]
    GIFTIMAGE_FIELD_NUMBER: _ClassVar[int]
    AGREEMSGID_FIELD_NUMBER: _ClassVar[int]
    COLORVALUELIST_FIELD_NUMBER: _ClassVar[int]
    common: Message
    user: User
    content: str
    visibleToSende: bool
    giftImage: Image
    agreeMsgId: int
    colorValueList: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, common: _Optional[_Union[Message, _Mapping]] = ..., user: _Optional[_Union[User, _Mapping]] = ..., content: _Optional[str] = ..., visibleToSende: bool = ..., giftImage: _Optional[_Union[Image, _Mapping]] = ..., agreeMsgId: _Optional[int] = ..., colorValueList: _Optional[_Iterable[str]] = ...) -> None: ...

class MatchAgainstScoreMessage(_message.Message):
    __slots__ = ["common", "against", "matchStatus", "displayStatus"]
    COMMON_FIELD_NUMBER: _ClassVar[int]
    AGAINST_FIELD_NUMBER: _ClassVar[int]
    MATCHSTATUS_FIELD_NUMBER: _ClassVar[int]
    DISPLAYSTATUS_FIELD_NUMBER: _ClassVar[int]
    common: Common
    against: Against
    matchStatus: int
    displayStatus: int
    def __init__(self, common: _Optional[_Union[Common, _Mapping]] = ..., against: _Optional[_Union[Against, _Mapping]] = ..., matchStatus: _Optional[int] = ..., displayStatus: _Optional[int] = ...) -> None: ...

class Against(_message.Message):
    __slots__ = ["leftName", "leftLogo", "leftGoal", "rightName", "rightLogo", "rightGoal", "timestamp", "version", "leftTeamId", "rightTeamId", "diffSei2absSecond", "finalGoalStage", "currentGoalStage", "leftScoreAddition", "rightScoreAddition", "leftGoalInt", "rightGoalInt"]
    LEFTNAME_FIELD_NUMBER: _ClassVar[int]
    LEFTLOGO_FIELD_NUMBER: _ClassVar[int]
    LEFTGOAL_FIELD_NUMBER: _ClassVar[int]
    RIGHTNAME_FIELD_NUMBER: _ClassVar[int]
    RIGHTLOGO_FIELD_NUMBER: _ClassVar[int]
    RIGHTGOAL_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    LEFTTEAMID_FIELD_NUMBER: _ClassVar[int]
    RIGHTTEAMID_FIELD_NUMBER: _ClassVar[int]
    DIFFSEI2ABSSECOND_FIELD_NUMBER: _ClassVar[int]
    FINALGOALSTAGE_FIELD_NUMBER: _ClassVar[int]
    CURRENTGOALSTAGE_FIELD_NUMBER: _ClassVar[int]
    LEFTSCOREADDITION_FIELD_NUMBER: _ClassVar[int]
    RIGHTSCOREADDITION_FIELD_NUMBER: _ClassVar[int]
    LEFTGOALINT_FIELD_NUMBER: _ClassVar[int]
    RIGHTGOALINT_FIELD_NUMBER: _ClassVar[int]
    leftName: str
    leftLogo: Image
    leftGoal: str
    rightName: str
    rightLogo: Image
    rightGoal: str
    timestamp: int
    version: int
    leftTeamId: int
    rightTeamId: int
    diffSei2absSecond: int
    finalGoalStage: int
    currentGoalStage: int
    leftScoreAddition: int
    rightScoreAddition: int
    leftGoalInt: int
    rightGoalInt: int
    def __init__(self, leftName: _Optional[str] = ..., leftLogo: _Optional[_Union[Image, _Mapping]] = ..., leftGoal: _Optional[str] = ..., rightName: _Optional[str] = ..., rightLogo: _Optional[_Union[Image, _Mapping]] = ..., rightGoal: _Optional[str] = ..., timestamp: _Optional[int] = ..., version: _Optional[int] = ..., leftTeamId: _Optional[int] = ..., rightTeamId: _Optional[int] = ..., diffSei2absSecond: _Optional[int] = ..., finalGoalStage: _Optional[int] = ..., currentGoalStage: _Optional[int] = ..., leftScoreAddition: _Optional[int] = ..., rightScoreAddition: _Optional[int] = ..., leftGoalInt: _Optional[int] = ..., rightGoalInt: _Optional[int] = ...) -> None: ...

class Common(_message.Message):
    __slots__ = ["method", "msgId", "roomId", "createTime", "monitor", "isShowMsg", "describe", "foldType", "anchorFoldType", "priorityScore", "logId", "msgProcessFilterK", "msgProcessFilterV", "user", "anchorFoldTypeV2", "processAtSeiTimeMs", "randomDispatchMs", "isDispatch", "channelId", "diffSei2absSecond", "anchorFoldDuration"]
    METHOD_FIELD_NUMBER: _ClassVar[int]
    MSGID_FIELD_NUMBER: _ClassVar[int]
    ROOMID_FIELD_NUMBER: _ClassVar[int]
    CREATETIME_FIELD_NUMBER: _ClassVar[int]
    MONITOR_FIELD_NUMBER: _ClassVar[int]
    ISSHOWMSG_FIELD_NUMBER: _ClassVar[int]
    DESCRIBE_FIELD_NUMBER: _ClassVar[int]
    FOLDTYPE_FIELD_NUMBER: _ClassVar[int]
    ANCHORFOLDTYPE_FIELD_NUMBER: _ClassVar[int]
    PRIORITYSCORE_FIELD_NUMBER: _ClassVar[int]
    LOGID_FIELD_NUMBER: _ClassVar[int]
    MSGPROCESSFILTERK_FIELD_NUMBER: _ClassVar[int]
    MSGPROCESSFILTERV_FIELD_NUMBER: _ClassVar[int]
    USER_FIELD_NUMBER: _ClassVar[int]
    ANCHORFOLDTYPEV2_FIELD_NUMBER: _ClassVar[int]
    PROCESSATSEITIMEMS_FIELD_NUMBER: _ClassVar[int]
    RANDOMDISPATCHMS_FIELD_NUMBER: _ClassVar[int]
    ISDISPATCH_FIELD_NUMBER: _ClassVar[int]
    CHANNELID_FIELD_NUMBER: _ClassVar[int]
    DIFFSEI2ABSSECOND_FIELD_NUMBER: _ClassVar[int]
    ANCHORFOLDDURATION_FIELD_NUMBER: _ClassVar[int]
    method: str
    msgId: int
    roomId: int
    createTime: int
    monitor: int
    isShowMsg: bool
    describe: str
    foldType: int
    anchorFoldType: int
    priorityScore: int
    logId: str
    msgProcessFilterK: str
    msgProcessFilterV: str
    user: User
    anchorFoldTypeV2: int
    processAtSeiTimeMs: int
    randomDispatchMs: int
    isDispatch: bool
    channelId: int
    diffSei2absSecond: int
    anchorFoldDuration: int
    def __init__(self, method: _Optional[str] = ..., msgId: _Optional[int] = ..., roomId: _Optional[int] = ..., createTime: _Optional[int] = ..., monitor: _Optional[int] = ..., isShowMsg: bool = ..., describe: _Optional[str] = ..., foldType: _Optional[int] = ..., anchorFoldType: _Optional[int] = ..., priorityScore: _Optional[int] = ..., logId: _Optional[str] = ..., msgProcessFilterK: _Optional[str] = ..., msgProcessFilterV: _Optional[str] = ..., user: _Optional[_Union[User, _Mapping]] = ..., anchorFoldTypeV2: _Optional[int] = ..., processAtSeiTimeMs: _Optional[int] = ..., randomDispatchMs: _Optional[int] = ..., isDispatch: bool = ..., channelId: _Optional[int] = ..., diffSei2absSecond: _Optional[int] = ..., anchorFoldDuration: _Optional[int] = ...) -> None: ...

class User(_message.Message):
    __slots__ = ["id", "shortId", "nickName", "gender", "Signature", "Level", "Birthday", "Telephone", "AvatarThumb", "AvatarMedium", "AvatarLarge", "Verified", "Experience", "city", "Status", "CreateTime", "ModifyTime", "Secret", "ShareQrcodeUri", "IncomeSharePercent", "BadgeImageList", "SpecialId", "AvatarBorder", "Medal", "RealTimeIconsList"]
    ID_FIELD_NUMBER: _ClassVar[int]
    SHORTID_FIELD_NUMBER: _ClassVar[int]
    NICKNAME_FIELD_NUMBER: _ClassVar[int]
    GENDER_FIELD_NUMBER: _ClassVar[int]
    SIGNATURE_FIELD_NUMBER: _ClassVar[int]
    LEVEL_FIELD_NUMBER: _ClassVar[int]
    BIRTHDAY_FIELD_NUMBER: _ClassVar[int]
    TELEPHONE_FIELD_NUMBER: _ClassVar[int]
    AVATARTHUMB_FIELD_NUMBER: _ClassVar[int]
    AVATARMEDIUM_FIELD_NUMBER: _ClassVar[int]
    AVATARLARGE_FIELD_NUMBER: _ClassVar[int]
    VERIFIED_FIELD_NUMBER: _ClassVar[int]
    EXPERIENCE_FIELD_NUMBER: _ClassVar[int]
    CITY_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    CREATETIME_FIELD_NUMBER: _ClassVar[int]
    MODIFYTIME_FIELD_NUMBER: _ClassVar[int]
    SECRET_FIELD_NUMBER: _ClassVar[int]
    SHAREQRCODEURI_FIELD_NUMBER: _ClassVar[int]
    INCOMESHAREPERCENT_FIELD_NUMBER: _ClassVar[int]
    BADGEIMAGELIST_FIELD_NUMBER: _ClassVar[int]
    SPECIALID_FIELD_NUMBER: _ClassVar[int]
    AVATARBORDER_FIELD_NUMBER: _ClassVar[int]
    MEDAL_FIELD_NUMBER: _ClassVar[int]
    REALTIMEICONSLIST_FIELD_NUMBER: _ClassVar[int]
    id: int
    shortId: int
    nickName: str
    gender: int
    Signature: str
    Level: int
    Birthday: int
    Telephone: str
    AvatarThumb: Image
    AvatarMedium: Image
    AvatarLarge: Image
    Verified: bool
    Experience: int
    city: str
    Status: int
    CreateTime: int
    ModifyTime: int
    Secret: int
    ShareQrcodeUri: str
    IncomeSharePercent: int
    BadgeImageList: _containers.RepeatedCompositeFieldContainer[Image]
    SpecialId: str
    AvatarBorder: Image
    Medal: Image
    RealTimeIconsList: _containers.RepeatedCompositeFieldContainer[Image]
    def __init__(self, id: _Optional[int] = ..., shortId: _Optional[int] = ..., nickName: _Optional[str] = ..., gender: _Optional[int] = ..., Signature: _Optional[str] = ..., Level: _Optional[int] = ..., Birthday: _Optional[int] = ..., Telephone: _Optional[str] = ..., AvatarThumb: _Optional[_Union[Image, _Mapping]] = ..., AvatarMedium: _Optional[_Union[Image, _Mapping]] = ..., AvatarLarge: _Optional[_Union[Image, _Mapping]] = ..., Verified: bool = ..., Experience: _Optional[int] = ..., city: _Optional[str] = ..., Status: _Optional[int] = ..., CreateTime: _Optional[int] = ..., ModifyTime: _Optional[int] = ..., Secret: _Optional[int] = ..., ShareQrcodeUri: _Optional[str] = ..., IncomeSharePercent: _Optional[int] = ..., BadgeImageList: _Optional[_Iterable[_Union[Image, _Mapping]]] = ..., SpecialId: _Optional[str] = ..., AvatarBorder: _Optional[_Union[Image, _Mapping]] = ..., Medal: _Optional[_Union[Image, _Mapping]] = ..., RealTimeIconsList: _Optional[_Iterable[_Union[Image, _Mapping]]] = ...) -> None: ...

class Image(_message.Message):
    __slots__ = ["urlListList", "uri", "height", "width", "avgColor", "imageType", "openWebUrl", "content", "isAnimated", "FlexSettingList", "TextSettingList"]
    URLLISTLIST_FIELD_NUMBER: _ClassVar[int]
    URI_FIELD_NUMBER: _ClassVar[int]
    HEIGHT_FIELD_NUMBER: _ClassVar[int]
    WIDTH_FIELD_NUMBER: _ClassVar[int]
    AVGCOLOR_FIELD_NUMBER: _ClassVar[int]
    IMAGETYPE_FIELD_NUMBER: _ClassVar[int]
    OPENWEBURL_FIELD_NUMBER: _ClassVar[int]
    CONTENT_FIELD_NUMBER: _ClassVar[int]
    ISANIMATED_FIELD_NUMBER: _ClassVar[int]
    FLEXSETTINGLIST_FIELD_NUMBER: _ClassVar[int]
    TEXTSETTINGLIST_FIELD_NUMBER: _ClassVar[int]
    urlListList: _containers.RepeatedScalarFieldContainer[str]
    uri: str
    height: int
    width: int
    avgColor: str
    imageType: int
    openWebUrl: str
    content: ImageContent
    isAnimated: bool
    FlexSettingList: NinePatchSetting
    TextSettingList: NinePatchSetting
    def __init__(self, urlListList: _Optional[_Iterable[str]] = ..., uri: _Optional[str] = ..., height: _Optional[int] = ..., width: _Optional[int] = ..., avgColor: _Optional[str] = ..., imageType: _Optional[int] = ..., openWebUrl: _Optional[str] = ..., content: _Optional[_Union[ImageContent, _Mapping]] = ..., isAnimated: bool = ..., FlexSettingList: _Optional[_Union[NinePatchSetting, _Mapping]] = ..., TextSettingList: _Optional[_Union[NinePatchSetting, _Mapping]] = ...) -> None: ...

class NinePatchSetting(_message.Message):
    __slots__ = ["settingListList"]
    SETTINGLISTLIST_FIELD_NUMBER: _ClassVar[int]
    settingListList: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, settingListList: _Optional[_Iterable[str]] = ...) -> None: ...

class ImageContent(_message.Message):
    __slots__ = ["name", "fontColor", "level", "alternativeText"]
    NAME_FIELD_NUMBER: _ClassVar[int]
    FONTCOLOR_FIELD_NUMBER: _ClassVar[int]
    LEVEL_FIELD_NUMBER: _ClassVar[int]
    ALTERNATIVETEXT_FIELD_NUMBER: _ClassVar[int]
    name: str
    fontColor: str
    level: int
    alternativeText: str
    def __init__(self, name: _Optional[str] = ..., fontColor: _Optional[str] = ..., level: _Optional[int] = ..., alternativeText: _Optional[str] = ...) -> None: ...

class PushFrame(_message.Message):
    __slots__ = ["seqId", "logId", "service", "method", "headersList", "payloadEncoding", "payloadType", "payload"]
    SEQID_FIELD_NUMBER: _ClassVar[int]
    LOGID_FIELD_NUMBER: _ClassVar[int]
    SERVICE_FIELD_NUMBER: _ClassVar[int]
    METHOD_FIELD_NUMBER: _ClassVar[int]
    HEADERSLIST_FIELD_NUMBER: _ClassVar[int]
    PAYLOADENCODING_FIELD_NUMBER: _ClassVar[int]
    PAYLOADTYPE_FIELD_NUMBER: _ClassVar[int]
    PAYLOAD_FIELD_NUMBER: _ClassVar[int]
    seqId: int
    logId: int
    service: int
    method: int
    headersList: _containers.RepeatedCompositeFieldContainer[HeadersList]
    payloadEncoding: str
    payloadType: str
    payload: bytes
    def __init__(self, seqId: _Optional[int] = ..., logId: _Optional[int] = ..., service: _Optional[int] = ..., method: _Optional[int] = ..., headersList: _Optional[_Iterable[_Union[HeadersList, _Mapping]]] = ..., payloadEncoding: _Optional[str] = ..., payloadType: _Optional[str] = ..., payload: _Optional[bytes] = ...) -> None: ...

class HeadersList(_message.Message):
    __slots__ = ["key", "value"]
    KEY_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    key: str
    value: str
    def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...



from biliup.common.util import client
from biliup.config import config
from . import logger, match1
from ..engine.decorators import Plugin
from ..engine.download import DownloadBase


@Plugin.download(regexp=r'https?://cc\.163\.com')
class CC(DownloadBase):
    def __init__(self, fname, url, suffix='flv'):
        super().__init__(fname, url, suffix)
        self.cc_protocol = config.get('cc_protocol', 'hls')

    async def acheck_stream(self, is_check=False):
        rid = match1(self.url, r"(\d{4,})")
        room_info = (await client.get(
            f"https://api.cc.163.com/v1/activitylives/anchor/lives?anchor_ccid={rid}",
            timeout=5,
            headers=self.fake_headers
        )).json()
        if len(room_info["data"][rid]) <= 1:
            logger.debug(f"{self.plugin_msg}: 未开播")
            return False

        if is_check:
            return True

        try:
            channel_id = room_info["data"][rid]["channel_id"]
            channel_info = (await client.get(
                f"https://cc.163.com/live/channel/?channelids={channel_id}",
                timeout=5,
                headers=self.fake_headers
            )).json()["data"][0]
            self.room_title = channel_info["title"]
            if self.cc_protocol == "hls":
                self.raw_stream_url = channel_info["sharefile"]
            else:
                original = {"vbr": 0}
                for level in channel_info["quickplay"]["resolution"].values():
                    original = level if level["vbr"] > original["vbr"] else original
                self.raw_stream_url = list(original["cdn"].values())[0]
        except:
            logger.exception(f"{self.plugin_msg}: ")
            return False

        return True



from enum import IntEnum

class EWebSocketCommandType(IntEnum):
    EWSCmd_NULL = 0
    EWSCmd_RegisterReq = 1
    EWSCmd_RegisterRsp = 2
    EWSCmd_WupReq = 3
    EWSCmd_WupRsp = 4
    EWSCmdC2S_HeartBeat = 5
    EWSCmdS2C_HeartBeatAck = 6
    EWSCmdS2C_MsgPushReq = 7
    EWSCmdC2S_DeregisterReq = 8
    EWSCmdS2C_DeRegisterRsp = 9
    EWSCmdC2S_VerifyCookieReq = 10
    EWSCmdS2C_VerifyCookieRsp = 11
    EWSCmdC2S_VerifyHuyaTokenReq = 12
    EWSCmdS2C_VerifyHuyaTokenRsp = 13



from functools import wraps
from biliup.common.tars import tarscore


STANDARD_CHARSET = 'utf-8'


def auto_decode_fields(cls):
    """自动解码类中的bytes类型数据"""
    original_read_from = cls.readFrom

    @staticmethod
    @wraps(original_read_from)
    def wrapped_read_from(ios: tarscore.TarsInputStream):
        value = original_read_from(ios)
        # 遍历对象的所有属性
        for attr_name, attr_value in vars(value).items():
            # 如果是bytes类型
            if isinstance(attr_value, bytes):
                # 将其解码为字符串
                setattr(value, attr_name, attr_value.decode(STANDARD_CHARSET))
        return value

    cls.readFrom = wrapped_read_from
    return cls


import asyncio
import base64
import hashlib
import json
import math
import os
import re
import sys
import time
import urllib.parse
from dataclasses import asdict, dataclass, field, InitVar
from json import JSONDecodeError
from os.path import splitext, basename
from typing import Union, Any, List, Optional, Callable
from urllib import parse
from urllib.parse import quote

import aiohttp
import requests.utils
import rsa
import xml.etree.ElementTree as ET
from requests.adapters import HTTPAdapter, Retry

from biliup.config import config
from ..engine import Plugin
from ..engine.upload import UploadBase, logger


@Plugin.upload(platform="bili_web")
class BiliWeb(UploadBase):
    def __init__(
        self,
        principal,
        data,
        user,
        user_cookie='cookies.json',
        submit_api: Optional[str] = 'web',
        copyright: int = 2,
        postprocessor: Optional[Callable] = None,
        dtime: Optional[int] = None,
        dynamic='',
        lines: Optional[str] = 'AUTO',
        threads: int = 3,
        tid: int = 122,
        tags: Optional[List[str]] = None,
        cover_path=None,
        description='',
        credits=[],
    ):
        """
        :param principal:
        :param data:
        :param user_cookie: 投稿用户cookie文件
        :param submit_api:
        :param copyright:
        :param postprocessor:
        :param dtime: 延时发布时间。需距离提交大于4小时，格式为10位时间戳
        :param dynamic:
        :param lines: 上传线路
        :param threads: 上传线程数
        :param tid: 稿件分区
        :param tags: 稿件标签
        :param cover_path: 稿件封面路径
        :param description: 视频简介
        :param credits: ???
        """
        super().__init__(principal, data, persistence_path='bili.cookie', postprocessor=postprocessor)
        if tags is None:
            tags = []
        else:
            tags = [str(tag).format(streamer=self.data['name']) for tag in tags]
        self.user = user # 旧版本的登录用户
        self.user_cookie = user_cookie # 新版本的登录用户cookie文件
        self.lines = lines
        self.submit_api = submit_api or 'web'
        self.threads = threads
        self.tid = tid
        self.tags = tags
        self.cover_path = cover_path
        self.desc = description
        self.credits = credits
        self.dynamic = dynamic
        self.copyright = copyright
        self.dtime = dtime

    def upload(
        self,
        file_list: List[UploadBase.FileInfo],
        database_row_id: int = 0
    ) -> List[UploadBase.FileInfo]:
        '''
        上传视频
        :param file_list: 视频文件名列表
        :param database_row_id: 数据库行ID
        :return: 上传结果
        '''
        logger.info(f"开始上传视频 {database_row_id}")
        video = Data()
        video.dynamic = self.dynamic
        with BiliBili(video) as bili:
            bili.app_key = self.user.get('app_key')
            bili.appsec = self.user.get('appsec')
            bili.login(self.persistence_path, self.user_cookie)
            for file in file_list:
                video_part = bili.upload_file(file.video, self.lines, self.threads)  # 上传视频
                video_part['title'] = video_part['title'][:80]
                video.append(video_part)  # 添加已经上传的视频
            video.title = self.data["format_title"][:80]  # 稿件标题限制80字
            if self.credits:
                video.desc_v2 = self.creditsToDesc_v2()
            else:
                video.desc_v2=[{
                    "raw_text": self.desc,
                    "biz_id": "",
                    "type": 1
                }]
            video.desc = self.desc
            video.copyright = self.copyright
            if self.copyright == 2:
                video.source = self.data["url"]  # 添加转载地址说明
            # 设置视频分区,默认为174 生活，其他分区
            video.tid = self.tid
            video.set_tag(self.tags)
            if self.dtime:
                video.delay_time(int(time.time()) + self.dtime)
            if self.cover_path:
                video.cover = bili.cover_up(self.cover_path).replace('http:', '')
            ret = bili.submit(self.submit_api)  # 提交视频
        logger.info(f"上传成功: {ret}")
        return file_list

    def creditsToDesc_v2(self):
            desc_v2 = []
            desc_v2_tmp = self.desc
            for credit in self.credits:
                try :
                    num = desc_v2_tmp.index("@credit")
                    desc_v2.append({
                        "raw_text": " "+desc_v2_tmp[:num],
                        "biz_id": "",
                        "type": 1
                    })
                    desc_v2.append({
                        "raw_text": credit["username"],
                        "biz_id": str(credit["uid"]),
                        "type": 2
                    })
                    self.desc = self.desc.replace(
                        "@credit", "@"+credit["username"]+"  ", 1)
                    desc_v2_tmp = desc_v2_tmp[num+7:]
                except IndexError:
                    logger.error('简介中的@credit占位符少于credits的数量,替换失败')
            desc_v2.append({
                "raw_text": " "+desc_v2_tmp,
                "biz_id": "",
                "type": 1
            })
            desc_v2[0]["raw_text"] = desc_v2[0]["raw_text"][1:]  # 开头空格会导致识别简介过长
            return desc_v2

class BiliBili:
    def __init__(self, video: 'Data'):
        self.app_key = None
        self.appsec = None
        if self.app_key is None or self.appsec is None:
            self.app_key = 'ae57252b0c09105d'
            self.appsec = 'c75875c596a69eb55bd119e74b07cfe3'
        self.__session = requests.Session()
        self.video = video
        self.__session.mount('https://', HTTPAdapter(max_retries=Retry(total=5)))
        self.__session.headers.update({
            'user-agent': "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 Chrome/63.0.3239.108",
            'referer': "https://www.bilibili.com/",
            'connection': 'keep-alive'
        })
        self.cookies = None
        self.access_token = None
        self.refresh_token = None
        self.account = None
        self.__bili_jct = None
        self._auto_os = None
        self.persistence_path = 'engine/bili.cookie'

    def check_tag(self, tag):
        r = self.__session.get("https://member.bilibili.com/x/vupre/web/topic/tag/check?tag=" + tag).json()
        if r["code"] == 0:
            return True
        else:
            return False

    def get_qrcode(self):
        params = {
            "appkey": "4409e2ce8ffd12b8",
            "local_id": "0",
            "ts": int(time.time()),
        }
        params["sign"] = hashlib.md5(
            f"{urllib.parse.urlencode(params)}59b43e04ad6965f34319062b478f83dd".encode()).hexdigest()
        response = self.__session.post("http://passport.bilibili.com/x/passport-tv-login/qrcode/auth_code", data=params,
                                       timeout=5)
        r = response.json()
        if r and r["code"] == 0:
            return r

    async def login_by_qrcode(self, value):
        params = {
            "appkey": "4409e2ce8ffd12b8",
            "auth_code": value["data"]["auth_code"],
            "local_id": "0",
            "ts": int(time.time()),
        }
        params["sign"] = hashlib.md5(
            f"{urllib.parse.urlencode(params)}59b43e04ad6965f34319062b478f83dd".encode()).hexdigest()
        for i in range(0, 120):
            await asyncio.sleep(1)
            response = self.__session.post("http://passport.bilibili.com/x/passport-tv-login/qrcode/poll", data=params,
                                           timeout=5)
            r = response.json()
            if r and r["code"] == 0:
                return r
        raise "Qrcode timeout"

    def tid_archive(self, cookies):
        requests.utils.add_dict_to_cookiejar(self.__session.cookies, cookies)
        response = self.__session.get("https://member.bilibili.com/x/vupre/web/archive/pre")
        return response.json()

    def myinfo(self, cookies):
        requests.utils.add_dict_to_cookiejar(self.__session.cookies, cookies)
        response = self.__session.get('http://api.bilibili.com/x/space/myinfo')
        return response.json()

    def login(self, persistence_path, user_cookie):
        self.persistence_path = user_cookie
        if os.path.isfile(self.persistence_path):
            print('使用持久化内容上传')
            self.load()
        if self.cookies:
            try:
                self.login_by_cookies(self.cookies)
            except:
                logger.exception('login error')
                self.login_by_password(**self.account)
        else:
            self.login_by_password(**self.account)
        self.store()

    def load(self):
        try:
            with open(self.persistence_path) as f:
                self.cookies = json.load(f)
                self.access_token = self.cookies['token_info']['access_token']
                self.refresh_token = self.cookies['token_info']['refresh_token']
        except (JSONDecodeError, KeyError):
            logger.exception('加载cookie出错')

    def store(self):
        with open(self.persistence_path, "w") as f:
            json.dump({**self.cookies,
                       'access_token': self.access_token,
                       'refresh_token': self.refresh_token
                       }, f)

    def send_sms(self, phone_number, country_code):
        params = {
            "actionKey": "appkey",
            "appkey": "783bbb7264451d82",
            "build": 6510400,
            "channel": "bili",
            "cid": country_code,
            "device": "phone",
            "mobi_app": "android",
            "platform": "android",
            "tel": phone_number,
            "ts": int(time.time()),
        }
        sign = hashlib.md5(f"{urllib.parse.urlencode(params)}2653583c8873dea268ab9386918b1d65".encode()).hexdigest()
        payload = f"{urllib.parse.urlencode(params)}&sign={sign}"
        response = self.__session.post("https://passport.bilibili.com/x/passport-login/sms/send", data=payload,
                                       timeout=5)
        return response.json()

    def login_by_sms(self, code, params):
        params["code"] = code
        params["sign"] = hashlib.md5(
            f"{urllib.parse.urlencode(params)}59b43e04ad6965f34319062b478f83dd".encode()).hexdigest()
        response = self.__session.post("https://passport.bilibili.com/x/passport-login/login/sms", data=params,
                                       timeout=5)
        r = response.json()
        if r and r["code"] == 0:
            return r

    def login_by_password(self, username, password):
        print('使用账号上传')
        key_hash, pub_key = self.get_key()
        encrypt_password = base64.b64encode(rsa.encrypt(f'{key_hash}{password}'.encode(), pub_key))
        payload = {
            "actionKey": 'appkey',
            "appkey": self.app_key,
            "build": 6270200,
            "captcha": '',
            "challenge": '',
            "channel": 'bili',
            "device": 'phone',
            "mobi_app": 'android',
            "password": encrypt_password,
            "permission": 'ALL',
            "platform": 'android',
            "seccode": "",
            "subid": 1,
            "ts": int(time.time()),
            "username": username,
            "validate": "",
        }
        response = self.__session.post("https://passport.bilibili.com/x/passport-login/oauth2/login", timeout=5,
                                       data={**payload, 'sign': self.sign(parse.urlencode(payload))})
        r = response.json()
        if r['code'] != 0 or r.get('data') is None or r['data'].get('cookie_info') is None:
            raise RuntimeError(r)
        try:
            for cookie in r['data']['cookie_info']['cookies']:
                self.__session.cookies.set(cookie['name'], cookie['value'])
                if 'bili_jct' == cookie['name']:
                    self.__bili_jct = cookie['value']
            self.cookies = self.__session.cookies.get_dict()
            self.access_token = r['data']['token_info']['access_token']
            self.refresh_token = r['data']['token_info']['refresh_token']
        except:
            raise RuntimeError(r)
        return r

    def login_by_cookies(self, cookie):
        logger.info(f'{self.__class__.__name__}: login by cookies')
        cookies_dict = {c['name']: c['value'] for c in cookie['cookie_info']['cookies']}
        requests.utils.add_dict_to_cookiejar(self.__session.cookies, cookies_dict)
        if 'bili_jct' in cookies_dict:
            self.__bili_jct = cookies_dict['bili_jct']
        data = self.__session.get("https://api.bilibili.com/x/web-interface/nav", timeout=5).json()
        if data["code"] != 0:
            raise Exception(data)
        print('使用cookies上传')

    def sign(self, param):
        return hashlib.md5(f"{param}{self.appsec}".encode()).hexdigest()

    def get_key(self):
        url = "https://passport.bilibili.com/x/passport-login/web/key"
        payload = {
            'appkey': f'{self.app_key}',
            'sign': self.sign(f"appkey={self.app_key}"),
        }
        response = self.__session.get(url, data=payload, timeout=5)
        r = response.json()
        if r and r["code"] == 0:
            return r['data']['hash'], rsa.PublicKey.load_pkcs1_openssl_pem(r['data']['key'].encode())

    def probe(self):
        ret = self.__session.get('https://member.bilibili.com/preupload?r=probe', timeout=5).json()
        logger.info(f"线路:{ret['lines']}")
        data, auto_os = None, None
        min_cost = 0
        if ret['probe'].get('get'):
            method = 'get'
        else:
            method = 'post'
            data = bytes(int(1024 * 0.1 * 1024))
        for line in ret['lines']:
            start = time.perf_counter()
            test = self.__session.request(method, f"https:{line['probe_url']}", data=data, timeout=30)
            cost = time.perf_counter() - start
            print(line['query'], cost)
            if test.status_code != 200:
                return
            if not min_cost or min_cost > cost:
                auto_os = line
                min_cost = cost
        auto_os['cost'] = min_cost
        return auto_os

    def upload_file(self, filepath: str, lines='AUTO', tasks=3):
        """上传本地视频文件,返回视频信息dict
        b站目前支持4种上传线路upos, kodo, gcs, bos
        gcs: {"os":"gcs","query":"bucket=bvcupcdngcsus&probe_version=20221109",
        "probe_url":"//storage.googleapis.com/bvcupcdngcsus/OK"},
        bos: {"os":"bos","query":"bucket=bvcupcdnboshb&probe_version=20221109",
        "probe_url":"??"}
        """
        preferred_upos_cdn = None
        if not self._auto_os:
            if lines == 'bda':
                self._auto_os = {"os": "upos", "query": "upcdn=bda&probe_version=20221109",
                                 "probe_url": "//upos-cs-upcdnbda.bilivideo.com/OK"}
                preferred_upos_cdn = 'bda'
            elif lines in {'bda2', 'cs-bda2'}:
                self._auto_os = {"os": "upos", "query": "upcdn=bda2&probe_version=20221109",
                                 "probe_url": "//upos-cs-upcdnbda2.bilivideo.com/OK"}
                preferred_upos_cdn = 'bda2'
            elif lines == 'ws':
                self._auto_os = {"os": "upos", "query": "upcdn=ws&probe_version=20221109",
                                 "probe_url": "//upos-cs-upcdnws.bilivideo.com/OK"}
                preferred_upos_cdn = 'ws'
            elif lines in {'qn', 'cs-qn'}:
                self._auto_os = {"os": "upos", "query": "upcdn=qn&probe_version=20221109",
                                 "probe_url": "//upos-cs-upcdnqn.bilivideo.com/OK"}
                preferred_upos_cdn = 'qn'
            elif lines == 'bldsa':
                self._auto_os = {"os": "upos", "query": "upcdn=bldsa&probe_version=20221109",
                                 "probe_url": "//upos-cs-upcdnbldsa.bilivideo.com/OK"}
                preferred_upos_cdn = 'bldsa'
            elif lines == 'tx':
                self._auto_os = {"os": "upos", "query": "upcdn=tx&probe_version=20221109",
                                 "probe_url": "//upos-cs-upcdntx.bilivideo.com/OK"}
                preferred_upos_cdn = 'tx'
            elif lines == 'txa':
                self._auto_os = {"os": "upos", "query": "upcdn=txa&probe_version=20221109",
                                 "probe_url": "//upos-cs-upcdntxa.bilivideo.com/OK"}
                preferred_upos_cdn = 'txa'
            else:
                self._auto_os = self.probe()
            logger.info(f"线路选择 => {self._auto_os['os']}: {self._auto_os['query']}. time: {self._auto_os.get('cost')}")
        if self._auto_os['os'] == 'upos':
            upload = self.upos
        # elif self._auto_os['os'] == 'cos':
        #     upload = self.cos
        # elif self._auto_os['os'] == 'cos-internal':
        #     upload = lambda *args, **kwargs: self.cos(*args, **kwargs, internal=True)
        # elif self._auto_os['os'] == 'kodo':
        #     upload = self.kodo
        else:
            logger.error(f"NoSearch:{self._auto_os['os']}")
            raise NotImplementedError(self._auto_os['os'])
        logger.info(f"os: {self._auto_os['os']}")
        total_size = os.path.getsize(filepath)
        with open(filepath, 'rb') as f:
            query = {
                'r': self._auto_os['os'] if self._auto_os['os'] != 'cos-internal' else 'cos',
                'profile': 'ugcupos/bup' if 'upos' == self._auto_os['os'] else "ugcupos/bupfetch",
                'ssl': 0,
                'version': '2.8.12',
                'build': 2081200,
                'name': f.name,
                'size': total_size,
            }
            resp = self.__session.get(
                f"https://member.bilibili.com/preupload?{self._auto_os['query']}", params=query,
                timeout=5)
            ret = resp.json()
            logger.debug(f"preupload: {ret}")
            if preferred_upos_cdn:
                original_endpoint: str = ret['endpoint']
                if re.match(r'//upos-(sz|cs)-upcdn(bda2|ws|qn)\.bilivideo\.com', original_endpoint):
                    if re.match(r'bda2|qn|ws', preferred_upos_cdn):
                        logger.debug(f"Preferred UpOS CDN: {preferred_upos_cdn}")
                        new_endpoint = re.sub(r'upcdn(bda2|qn|ws)', f'upcdn{preferred_upos_cdn}', original_endpoint)
                        logger.debug(f"{original_endpoint} => {new_endpoint}")
                        ret['endpoint'] = new_endpoint
                    else:
                        logger.error(f"Unrecognized preferred_upos_cdn: {preferred_upos_cdn}")
                else:
                    logger.warning(f"Assigned UpOS endpoint {original_endpoint} was never seen before, something else might have changed, so will not modify it")
            return asyncio.run(upload(f, total_size, ret, tasks=tasks))

    async def cos(self, file, total_size, ret, chunk_size=10485760, tasks=3, internal=False):
        filename = file.name
        url = ret["url"]
        if internal:
            url = url.replace("cos.accelerate", "cos-internal.ap-shanghai")
        biz_id = ret["biz_id"]
        post_headers = {
            "Authorization": ret["post_auth"],
        }
        put_headers = {
            "Authorization": ret["put_auth"],
        }

        initiate_multipart_upload_result = ET.fromstring(self.__session.post(f'{url}?uploads&output=json', timeout=5,
                                                                             headers=post_headers).content)
        upload_id = initiate_multipart_upload_result.find('UploadId').text
        # 开始上传
        parts = []  # 分块信息
        chunks = math.ceil(total_size / chunk_size)  # 获取分块数量

        async def upload_chunk(session, chunks_data, params):
            async with session.put(url, params=params, raise_for_status=True,
                                   data=chunks_data, headers=put_headers) as r:
                end = time.perf_counter() - start
                parts.append({"Part": {"PartNumber": params['chunk'] + 1, "ETag": r.headers['Etag']}})
                sys.stdout.write(f"\r{params['end'] / 1000 / 1000 / end:.2f}MB/s "
                                 f"=> {params['partNumber'] / chunks:.1%}")

        start = time.perf_counter()
        await self._upload({
            'uploadId': upload_id,
            'chunks': chunks,
            'total': total_size
        }, file, chunk_size, upload_chunk, tasks=tasks)
        cost = time.perf_counter() - start
        fetch_headers = {
            "X-Upos-Fetch-Source": ret["fetch_headers"]["X-Upos-Fetch-Source"],
            "X-Upos-Auth": ret["fetch_headers"]["X-Upos-Auth"],
            "Fetch-Header-Authorization": ret["fetch_headers"]["Fetch-Header-Authorization"]
        }
        parts = sorted(parts, key=lambda x: x['Part']['PartNumber'])
        complete_multipart_upload = ET.Element('CompleteMultipartUpload')
        for part in parts:
            part_et = ET.SubElement(complete_multipart_upload, 'Part')
            part_number = ET.SubElement(part_et, 'PartNumber')
            part_number.text = str(part['Part']['PartNumber'])
            e_tag = ET.SubElement(part_et, 'ETag')
            e_tag.text = part['Part']['ETag']
        xml = ET.tostring(complete_multipart_upload)
        ii = 0
        while ii <= 3:
            try:
                res = self.__session.post(url, params={'uploadId': upload_id}, data=xml, headers=post_headers,
                                          timeout=15)
                if res.status_code == 200:
                    break
                raise IOError(res.text)
            except IOError:
                ii += 1
                logger.info("请求合并分片出现问题，尝试重连，次数：" + str(ii))
                time.sleep(15)
        ii = 0
        while ii <= 3:
            try:
                res = self.__session.post("https:" + ret["fetch_url"], headers=fetch_headers, timeout=15).json()
                if res.get('OK') == 1:
                    logger.info(f'{filename} uploaded >> {total_size / 1000 / 1000 / cost:.2f}MB/s. {res}')
                    return {"title": splitext(filename)[0], "filename": ret["bili_filename"], "desc": ""}
                raise IOError(res)
            except IOError:
                ii += 1
                logger.info("上传出现问题，尝试重连，次数：" + str(ii))
                time.sleep(15)

    async def kodo(self, file, total_size, ret, chunk_size=4194304, tasks=3):
        filename = file.name
        bili_filename = ret['bili_filename']
        key = ret['key']
        endpoint = f"https:{ret['endpoint']}"
        token = ret['uptoken']
        fetch_url = ret['fetch_url']
        fetch_headers = ret['fetch_headers']
        url = f'{endpoint}/mkblk'
        headers = {
            'Authorization': f"UpToken {token}",
        }
        # 开始上传
        parts = []  # 分块信息
        chunks = math.ceil(total_size / chunk_size)  # 获取分块数量

        async def upload_chunk(session, chunks_data, params):
            async with session.post(f'{url}/{len(chunks_data)}',
                                    data=chunks_data, headers=headers) as response:
                end = time.perf_counter() - start
                ctx = await response.json()
                parts.append({"index": params['chunk'], "ctx": ctx['ctx']})
                sys.stdout.write(f"\r{params['end'] / 1000 / 1000 / end:.2f}MB/s "
                                 f"=> {params['partNumber'] / chunks:.1%}")

        start = time.perf_counter()
        await self._upload({}, file, chunk_size, upload_chunk, tasks=tasks)
        cost = time.perf_counter() - start

        logger.info(f'{filename} uploaded >> {total_size / 1000 / 1000 / cost:.2f}MB/s')
        parts.sort(key=lambda x: x['index'])
        self.__session.post(f"{endpoint}/mkfile/{total_size}/key/{base64.urlsafe_b64encode(key.encode()).decode()}",
                            data=','.join(map(lambda x: x['ctx'], parts)), headers=headers, timeout=10)
        r = self.__session.post(f"https:{fetch_url}", headers=fetch_headers, timeout=5).json()
        if r["OK"] != 1:
            raise Exception(r)
        return {"title": splitext(filename)[0], "filename": bili_filename, "desc": ""}

    async def upos(self, file, total_size, ret, tasks=3):
        filename = file.name
        chunk_size = ret['chunk_size']
        auth = ret["auth"]
        endpoint = ret["endpoint"]
        biz_id = ret["biz_id"]
        upos_uri = ret["upos_uri"]
        url = f"https:{endpoint}/{upos_uri.replace('upos://', '')}"  # 视频上传路径
        headers = {
            "X-Upos-Auth": auth
        }
        # 向上传地址申请上传，得到上传id等信息
        upload_id = self.__session.post(f'{url}?uploads&output=json', timeout=15,
                                        headers=headers).json()["upload_id"]
        # 开始上传
        parts = []  # 分块信息
        chunks = math.ceil(total_size / chunk_size)  # 获取分块数量

        async def upload_chunk(session, chunks_data, params):
            async with session.put(url, params=params, raise_for_status=True,
                                   data=chunks_data, headers=headers):
                end = time.perf_counter() - start
                parts.append({"partNumber": params['chunk'] + 1, "eTag": "etag"})
                sys.stdout.write(f"\r{params['end'] / 1000 / 1000 / end:.2f}MB/s "
                                 f"=> {params['partNumber'] / chunks:.1%}")

        start = time.perf_counter()
        await self._upload({
            'uploadId': upload_id,
            'chunks': chunks,
            'total': total_size
        }, file, chunk_size, upload_chunk, tasks=tasks)
        cost = time.perf_counter() - start
        p = {
            'name': filename,
            'uploadId': upload_id,
            'biz_id': biz_id,
            'output': 'json',
            'profile': 'ugcupos/bup'
        }
        attempt = 0
        while attempt <= 5:  # 一旦放弃就会丢失前面所有的进度，多试几次吧
            try:
                r = self.__session.post(url, params=p, json={"parts": parts}, headers=headers, timeout=15).json()
                if r.get('OK') == 1:
                    logger.info(f'{filename} uploaded >> {total_size / 1000 / 1000 / cost:.2f}MB/s. {r}')
                    return {"title": splitext(filename)[0], "filename": splitext(basename(upos_uri))[0], "desc": ""}
                raise IOError(r)
            except IOError:
                attempt += 1
                logger.info(f"请求合并分片时出现问题，尝试重连，次数：" + str(attempt))
                time.sleep(15)

    @staticmethod
    async def _upload(params, file, chunk_size, afunc, tasks=3):
        params['chunk'] = -1

        async def upload_chunk():
            while True:
                chunks_data = file.read(chunk_size)
                if not chunks_data:
                    return
                params['chunk'] += 1
                params['size'] = len(chunks_data)
                params['partNumber'] = params['chunk'] + 1
                params['start'] = params['chunk'] * chunk_size
                params['end'] = params['start'] + params['size']
                clone = params.copy()
                for i in range(10):
                    try:
                        await afunc(session, chunks_data, clone)
                        break
                    except (asyncio.TimeoutError, aiohttp.ClientError) as e:
                        logger.error(f"retry chunk{clone['chunk']} >> {i + 1}. {e}")

        async with aiohttp.ClientSession() as session:
            await asyncio.gather(*[upload_chunk() for _ in range(tasks)])

    def submit(self, submit_api: Optional[str] = 'web'):
        if not self.video.title:
            self.video.title = self.video.videos[0]["title"]
        self.__session.get('https://member.bilibili.com/x/geetest/pre/add', timeout=5)

        if submit_api is None:
            total_info = self.__session.get('http://api.bilibili.com/x/space/myinfo', timeout=15).json()
            if total_info.get('data') is None:
                logger.error(total_info)
            total_info = total_info.get('data')
            if total_info['level'] > 3 and total_info['follower'] > 1000:
                user_weight = 2
            else:
                user_weight = 1
            logger.info(f'用户权重: {user_weight}')
            submit_api = 'web'
        ret = None
        if submit_api == 'web':
            ret = self.submit_web()
            if ret["code"] != 0:
                logger.error(f'网页端接口提交失败: {ret}')
                raise Exception(ret)
        if not ret:
            raise Exception(f'不存在的选项：{submit_api}')
        return ret

    def submit_web(self):
        logger.info('使用网页端api提交')
        return self.__session.post(f'https://member.bilibili.com/x/vu/web/add?csrf={self.__bili_jct}', timeout=5,
                                   json=asdict(self.video)).json()

    def submit_client(self):
        logger.info('使用客户端api端提交')
        if not self.access_token:
            if self.account is None:
                raise RuntimeError("Access token is required, but account and access_token does not exist!")
            self.login_by_password(**self.account)
            self.store()
        while True:
            ret = self.__session.post(f'http://member.bilibili.com/x/vu/client/add?access_key={self.access_token}',
                                      timeout=5, json=asdict(self.video)).json()
            if ret['code'] == -101:
                logger.info(f'刷新token{ret}')
                self.login_by_password(**config['user']['account'])
                self.store()
                continue
            return ret

    def cover_up(self, img: str):
        """
        :param img: img path or stream
        :return: img URL
        """
        from PIL import Image
        from io import BytesIO

        with Image.open(img) as im:
            # 宽和高,需要16：10
            xsize, ysize = im.size
            if xsize / ysize > 1.6:
                delta = xsize - ysize * 1.6
                region = im.crop((delta / 2, 0, xsize - delta / 2, ysize))
            else:
                delta = ysize - xsize * 10 / 16
                region = im.crop((0, delta / 2, xsize, ysize - delta / 2))
            buffered = BytesIO()
            region.save(buffered, format=im.format)
        r = self.__session.post(
            url='https://member.bilibili.com/x/vu/web/cover/up',
            data={
                'cover': b'data:image/jpeg;base64,' + (base64.b64encode(buffered.getvalue())),
                'csrf': self.__bili_jct
            }, timeout=30
        )
        buffered.close()
        res = r.json()
        if res.get('data') is None:
            raise Exception(res)
        return res['data']['url']

    def get_tags(self, upvideo, typeid="", desc="", cover="", groupid=1, vfea=""):
        """
        上传视频后获得推荐标签
        :param vfea:
        :param groupid:
        :param typeid:
        :param desc:
        :param cover:
        :param upvideo:
        :return: 返回官方推荐的tag
        """
        url = f'https://member.bilibili.com/x/web/archive/tags?' \
              f'typeid={typeid}&title={quote(upvideo["title"])}&filename=filename&desc={desc}&cover={cover}' \
              f'&groupid={groupid}&vfea={vfea}'
        return self.__session.get(url=url, timeout=5).json()

    def __enter__(self):
        return self

    def __exit__(self, e_t, e_v, t_b):
        self.close()

    def close(self):
        """Closes all adapters and as such the session"""
        self.__session.close()


@dataclass
class Data:
    """
    cover: 封面图片，可由recovers方法得到视频的帧截图
    """
    copyright: int = 2
    source: str = ''
    tid: int = 21
    cover: str = ''
    title: str = ''
    desc_format_id: int = 0
    desc: str = ''
    dynamic: str = ''
    subtitle: dict = field(init=False)
    tag: Union[list, str] = ''
    videos: list = field(default_factory=list)
    dtime: Any = None
    open_subtitle: InitVar[bool] = False

    # interactive: int = 0
    # no_reprint: int 1
    # charging_pay: int 1

    def __post_init__(self, open_subtitle):
        self.subtitle = {"open": int(open_subtitle), "lan": ""}
        if self.dtime and self.dtime - int(time.time()) <= 14400:
            self.dtime = None
        if isinstance(self.tag, list):
            self.tag = ','.join(self.tag)

    def delay_time(self, dtime: int):
        """设置延时发布时间，距离提交大于2小时，格式为10位时间戳"""
        if dtime - int(time.time()) > 7200:
            self.dtime = dtime

    def set_tag(self, tag: list):
        """设置标签，tag为数组"""
        self.tag = ','.join(tag)

    def append(self, video):
        self.videos.append(video)



from typing import Optional
from urllib.parse import unquote, urlparse, parse_qs, urlencode, urlunparse

import requests
import random

from ..common.util import client
from ..config import config
from ..Danmaku import DanmakuClient
from ..common.abogus import ABogus
from ..engine.decorators import Plugin
from ..engine.download import DownloadBase
from . import logger, match1, random_user_agent, json_loads, test_jsengine


@Plugin.download(regexp=r'https?://(?:(?:www|m|live|v)\.)?douyin\.com')
class Douyin(DownloadBase):
    def __init__(self, fname, url, suffix='flv'):
        super().__init__(fname, url, suffix)
        self.douyin_danmaku = config.get('douyin_danmaku', False)
        self.douyin_quality = config.get('douyin_quality', 'origin')
        self.douyin_protocol = config.get('douyin_protocol', 'flv')
        self.douyin_double_screen = config.get('douyin_double_screen', False)
        self.douyin_true_origin = config.get('douyin_true_origin', False)
        self.__web_rid = None # 网页端房间号 或 抖音号
        self.__room_id = None # 单场直播的直播房间
        self.__sec_uid = None

    async def acheck_stream(self, is_check=False):

        self.fake_headers['user-agent'] = DouyinUtils.DOUYIN_USER_AGENT
        self.fake_headers['referer'] = "https://live.douyin.com/"

        self.fake_headers['cookie'] = config.get('user', {}).get('douyin_cookie', '')
        if self.fake_headers['cookie'] != "" and not self.fake_headers['cookie'].endswith(';'):
            self.fake_headers['cookie'] += ";"
        if "ttwid" not in self.fake_headers['cookie']:
            self.fake_headers['cookie'] += f'ttwid={DouyinUtils.get_ttwid()};'
        if 'odin_ttid=' not in self.fake_headers['cookie']:
            self.fake_headers['cookie'] += f"odin_ttid={DouyinUtils.generate_odin_ttid()};"
        if '__ac_nonce=' not in self.fake_headers['cookie']:
            self.fake_headers['cookie'] += f"__ac_nonce={DouyinUtils.generate_nonce()};"


        if "v.douyin" in self.url:
            try:
                resp = await client.get(self.url, headers=self.fake_headers, follow_redirects=False)
            except:
                return False
            try:
                if resp.status_code not in {301, 302}:
                    raise
                next_url = str(resp.next_request.url)
                if "webcast.amemv" in next_url:
                    self.__sec_uid = match1(next_url, r"sec_user_id=(.*?)&")
                    self.__room_id = match1(next_url.split("?")[0], r"(\d+)")
                elif "isedouyin.com/share/user" in next_url:
                    self.__sec_uid = match1(next_url, r"sec_uid=(.*?)&")
                else:
                    raise
            except:
                logger.error(f"{self.plugin_msg}: 不支持的链接")
                return False
        elif "/user/" in self.url:
            sec_uid = self.url.split("user/")[1].split("?")[0]
            if len(sec_uid) in {55, 76}:
                self.__sec_uid = sec_uid
            else:
                try:
                    user_page = (await client.get(self.url, headers=self.fake_headers)).text
                    user_page_data = unquote(
                        user_page.split('<script id="RENDER_DATA" type="application/json">')[1].split('</script>')[0])
                    web_rid = match1(user_page_data, r'"web_rid":"([^"]+)"')
                    if not web_rid:
                        logger.debug(f"{self.plugin_msg}: 未开播")
                        return False
                    self.__web_rid = web_rid
                except (KeyError, IndexError):
                    logger.error(f"{self.plugin_msg}: 房间号获取失败，请检查Cookie设置")
                    return False
                except:
                    logger.exception(f"{self.plugin_msg}: 房间号获取失败")
                    return False
        else:
            web_rid = self.url.split('douyin.com/')[1].split('/')[0].split('?')[0]
            if web_rid[0] == "+":
                web_rid = web_rid[1:]
            self.__web_rid = web_rid

        try:
            _room_info = {}
            if self.__web_rid:
                _room_info = await self.get_web_room_info(self.__web_rid)
                if _room_info:
                    if not _room_info['data'].get('user'):
                        if _room_info['data'].get('prompts', '') == '直播已结束':
                            return False
                        # 可能是用户被封禁
                        raise Exception(f"{str(_room_info)}")
                    self.__sec_uid = _room_info['data']['user']['sec_uid']
            # PCWeb 端无流 或 没有提供 web_rid
            if not _room_info.get('data', {}).get('data'):
                _room_info = await self.get_h5_room_info(self.__sec_uid, self.__room_id)
                if _room_info['data'].get('room', {}).get('owner'):
                    self.__web_rid = _room_info['data']['room']['owner']['web_rid']
            try:
                # 出现异常不用提示，直接到 移动网页 端获取
                room_info = _room_info['data']['data'][0]
            except (KeyError, IndexError):
                # 如果 移动网页 端也没有数据，当做未开播处理
                room_info = _room_info['data'].get('room', {})
                # 当做未开播处理
                # if not room_info:
                #     logger.info(f"{self.plugin_msg}: 获取直播间信息失败 {_room_info}")
            if room_info.get('status') != 2:
                logger.debug(f"{self.plugin_msg}: 未开播")
                return False
            self.__room_id = room_info['id_str']
            self.room_title = room_info['title']
        except:
            logger.exception(f"{self.plugin_msg}: 获取直播间信息失败")
            return False

        if is_check:
            return True
        else:
            # 清理上一次获取的直播流
            self.raw_stream_url = ""

        try:
            pull_data = room_info['stream_url']['live_core_sdk_data']['pull_data']
            if room_info['stream_url'].get('pull_datas') and self.douyin_double_screen:
                pull_data = next(iter(room_info['stream_url']['pull_datas'].values()))
            stream_data = json_loads(pull_data['stream_data'])['data']
        except:
            logger.exception(f"{self.plugin_msg}: 加载直播流失败")
            logger.debug(f"{self.plugin_msg}: room_info {room_info}")
            return False

        # 抖音FLV真原画
        if (
            self.douyin_true_origin  # 开启真原画
            and
            self.douyin_quality == 'origin' # 请求原画
            and
            self.douyin_protocol == 'flv' # 请求FLV
            # and
            # self.raw_stream_url.find('_or4.flv') != -1 # or4(origin)
        ):
            try:
                self.raw_stream_url = stream_data['ao']['main']['flv'].replace('&only_audio=1', '')
            except KeyError:
                logger.debug(f"{self.plugin_msg}: 未找到 ao 流 {stream_data}")

        if not self.raw_stream_url:
            # 原画origin 蓝光uhd 超清hd 高清sd 标清ld 流畅md 仅音频ao
            quality_items = ['origin', 'uhd', 'hd', 'sd', 'ld', 'md']
            quality = self.douyin_quality
            if quality not in quality_items:
                quality = quality_items[0]
            try:
                # 如果没有这个画质则取相近的 优先低清晰度
                if quality not in stream_data:
                    # 可选的清晰度 含自身
                    optional_quality_items = [x for x in quality_items if x in stream_data.keys() or x == quality]
                    # 自身在可选清晰度的位置
                    optional_quality_index = optional_quality_items.index(quality)
                    # 自身在所有清晰度的位置
                    quality_index = quality_items.index(quality)
                    # 高清晰度偏移
                    quality_left_offset = None
                    # 低清晰度偏移
                    quality_right_offset = None

                    if optional_quality_index + 1 < len(optional_quality_items):
                        quality_right_offset = quality_items.index(
                            optional_quality_items[optional_quality_index + 1]) - quality_index

                    if optional_quality_index - 1 >= 0:
                        quality_left_offset = quality_index - quality_items.index(
                            optional_quality_items[optional_quality_index - 1])

                    # 取相邻的清晰度
                    if quality_right_offset <= quality_left_offset:
                        quality = optional_quality_items[optional_quality_index + 1]
                    else:
                        quality = optional_quality_items[optional_quality_index - 1]

                protocol = 'hls' if self.douyin_protocol == 'hls' else 'flv'
                self.raw_stream_url = stream_data[quality]['main'][protocol]
            except:
                logger.exception(f"{self.plugin_msg}: 寻找清晰度失败")
                return False

        self.raw_stream_url = self.raw_stream_url.replace('http://', 'https://')
        return True

    def danmaku_init(self):
        if self.douyin_danmaku:
            if (js_runable := test_jsengine()):
                content = {
                    'web_rid': self.__web_rid,
                    'sec_uid': self.__sec_uid,
                    'room_id': self.__room_id,
                }
                self.danmaku = DanmakuClient(self.url, self.gen_download_filename(), content)
            else:
                logger.error(f"如需录制抖音弹幕，请至少安装一个 Javascript 解释器。如 pip install quickjs")

    async def get_web_room_info(self, web_rid: str) -> dict:
        query = {
            'app_name': 'douyin_web',
            # 'enter_from': random.choice(['link_share', 'web_live']),
            'enter_from': 'web_live',
            'live_id': '1',
            'web_rid': web_rid,
            'is_need_double_stream': "false"
        }
        target_url = DouyinUtils.build_request_url(f"https://live.douyin.com/webcast/room/web/enter/", query)
        logger.debug(f"{self.plugin_msg}: get_web_room_info {target_url}")
        web_info = await client.get(target_url, headers=self.fake_headers)
        web_info = json_loads(web_info.text)
        logger.debug(f"{self.plugin_msg}: get_web_room_info {web_info}")
        return web_info

    async def get_h5_room_info(self, sec_user_id: str, room_id: str) -> dict:
        '''
        Mobile web 的 API 信息，海外可能不允许使用
        '''
        if not sec_user_id:
            raise ValueError("sec_user_id is None")
        query = {
            'type_id': 0,
            'live_id': 1,
            'version_code': '99.99.99',
            'app_id': 1128,
            'room_id': room_id if room_id else 2, # 必要但不校验
            'sec_user_id': sec_user_id
        }
        abogus = ABogus(user_agent=DouyinUtils.DOUYIN_USER_AGENT)
        query_str, _, _, _ = abogus.generate_abogus(params=urlencode(query, doseq=True), body="")
        # target_url = DouyinUtils.build_request_url(f"https://live.douyin.com/webcast/room/web/enter/", query)
        info = await client.get(
            f"https://webcast.amemv.com/webcast/room/reflow/info/?{query_str}",
            headers=self.fake_headers
        )
        info = json_loads(info.text)
        logger.debug(f"{self.plugin_msg}: get_h5_room_info {info}")
        return info



class DouyinUtils:
    # 抖音ttwid
    _douyin_ttwid: Optional[str] = None
    # DOUYIN_USER_AGENT = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.159 Safari/537.36'
    DOUYIN_USER_AGENT = random_user_agent()
    DOUYIN_HTTP_HEADERS = {
        'user-agent': DOUYIN_USER_AGENT
    }
    CHARSET = "abcdef0123456789"
    LONG_CHATSET = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789_-"

    @staticmethod
    def get_ttwid() -> Optional[str]:
            if not DouyinUtils._douyin_ttwid:
                page = requests.get("https://live.douyin.com/1-2-3-4-5-6-7-8-9-0", timeout=15)
                DouyinUtils._douyin_ttwid = page.cookies.get("ttwid")
            return DouyinUtils._douyin_ttwid


    @staticmethod
    def generate_ms_token() -> str:
        '''生成随机 msToken'''
        return ''.join(random.choice(DouyinUtils.LONG_CHATSET) for _ in range(184))


    @staticmethod
    def generate_nonce() -> str:
        """生成 21 位随机十六进制小写 nonce"""
        return ''.join(random.choice(DouyinUtils.CHARSET) for _ in range(21))


    @staticmethod
    def generate_odin_ttid() -> str:
        """生成 160 位随机十六进制小写 odin_ttid"""
        return ''.join(random.choice(DouyinUtils.CHARSET) for _ in range(160))


    @staticmethod
    def build_request_url(url: str, query: Optional[dict] = None) -> str:
        # NOTE: 不能在类级别初始化，否则非首次生成的 abogus 有问题，原因未知
        abogus = ABogus(user_agent=DouyinUtils.DOUYIN_USER_AGENT)
        parsed_url = urlparse(url)
        existing_params = query or parse_qs(parsed_url.query)
        existing_params['aid'] = ['6383']
        existing_params['compress'] = ['gzip']
        existing_params['device_platform'] = ['web']
        existing_params['browser_language'] = ['zh-CN']
        existing_params['browser_platform'] = ['Win32']
        existing_params['browser_name'] = [DouyinUtils.DOUYIN_USER_AGENT.split('/')[0]]
        existing_params['browser_version'] = [DouyinUtils.DOUYIN_USER_AGENT.split(existing_params['browser_name'][0])[-1][1:]]
        if 'msToken' not in existing_params:
            existing_params['msToken'] = [DouyinUtils.generate_ms_token()]
        new_query_string = urlencode(existing_params, doseq=True)
        signed_query_string, _, _, _ = abogus.generate_abogus(params=new_query_string, body="")
        new_url = urlunparse((
            parsed_url.scheme,
            parsed_url.netloc,
            parsed_url.path,
            parsed_url.params,
            signed_query_string,
            parsed_url.fragment
        ))
        return new_url



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# filename: __timeQueue.py

# Tencent is pleased to support the open source community by making Tars available.
#
# Copyright (C) 2016THL A29 Limited, a Tencent company. All rights reserved.
#
# Licensed under the BSD 3-Clause License (the "License"); you may not use this file except
# in compliance with the License. You may obtain a copy of the License at
#
# https://opensource.org/licenses/BSD-3-Clause
#
# Unless required by applicable law or agreed to in writing, software distributed
# under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
# CONDITIONS OF ANY KIND, either express or implied. See the License for the
# specific language governing permissions and limitations under the License.
#

'''
@version: 0.01
@brief: 请求响应报文和超时队列
'''

import struct
import threading
import time

from .__logger import tarsLogger
from .__packet import RequestPacket
from .__packet import ResponsePacket
from .__tars import TarsInputStream
from .__tars import TarsOutputStream
from .__util import (NewLock, LockGuard)


class ReqMessage:
    '''
    @brief: 请求响应报文，保存一个请求响应所需要的数据
    '''
    SYNC_CALL = 1
    ASYNC_CALL = 2
    ONE_WAY = 3

    def __init__(self):
        self.type = ReqMessage.SYNC_CALL
        self.servant = None
        self.lock = None
        self.adapter = None
        self.request = None
        self.response = None
        self.callback = None
        self.begtime = None
        self.endtime = None
        self.isHash = False
        self.isConHash = False
        self.hashCode = 0

    def packReq(self):
        '''
        @brief: 序列化请求报文
        @return: 序列化后的请求报文
        @rtype: str
        '''
        if not self.request:
            return ''
        oos = TarsOutputStream()
        RequestPacket.writeTo(oos, self.request)
        reqpkt = oos.getBuffer()
        plen = len(reqpkt) + 4
        reqpkt = struct.pack('!i', plen) + reqpkt
        return reqpkt

    @staticmethod
    def unpackRspList(buf):
        '''
        @brief: 解码响应报文
        @param buf: 多个序列化后的响应报文数据
        @type buf: str
        @return: 解码出来的响应报文和解码的buffer长度
        @rtype: rsplist: 装有ResponsePacket的list
                unpacklen: int
        '''
        rsplist = []
        if not buf:
            return rsplist

        unpacklen = 0
        buf = buffer(buf)
        while True:
            if len(buf) - unpacklen < 4:
                break
            packsize = buf[unpacklen: unpacklen+4]
            packsize, = struct.unpack_from('!i', packsize)
            if len(buf) < unpacklen + packsize:
                break

            ios = TarsInputStream(buf[unpacklen+4: unpacklen+packsize])
            rsp = ResponsePacket.readFrom(ios)
            rsplist.append(rsp)
            unpacklen += packsize

        return rsplist, unpacklen

# 超时队列，加锁，线程安全


class TimeoutQueue:
    '''
    @brief: 超时队列，加锁，线程安全
            可以像队列一样FIFO，也可以像字典一样按key取item
    @todo: 限制队列长度
    '''

    def __init__(self, timeout=3):
        self.__uniqId = 0
        # self.__lock = threading.Lock()
        self.__lock = NewLock()
        self.__data = {}
        self.__queue = []
        self.__timeout = timeout

    def getTimeout(self):
        '''
        @brief: 获取超时时间，单位为s
        @return: 超时时间
        @rtype: float
        '''
        return self.__timeout

    def setTimeout(self, timeout):
        '''
        @brief: 设置超时时间，单位为s
        @param timeout: 超时时间
        @type timeout: float
        @return: None
        @rtype: None
        '''
        self.__timeout = timeout

    def size(self):
        '''
        @brief: 获取队列长度
        @return: 队列长度
        @rtype: int
        '''
        # self.__lock.acquire()
        lock = LockGuard(self.__lock)
        ret = len(self.__data)
        # self.__lock.release()
        return ret

    def generateId(self):
        '''
        @brief: 生成唯一id，0 < id < 2 ** 32
        @return: id
        @rtype: int
        '''
        # self.__lock.acquire()
        lock = LockGuard(self.__lock)
        ret = self.__uniqId
        ret = (ret + 1) % 0x7FFFFFFF
        while ret <= 0:
            ret = (ret + 1) % 0x7FFFFFFF
        self.__uniqId = ret
        # self.__lock.release()
        return ret

    def pop(self, uniqId=0, erase=True):
        '''
        @brief: 弹出item
        @param uniqId: item的id，如果为0，按FIFO弹出
        @type uniqId: int
        @param erase: 弹出后是否从字典里删除item
        @type erase: bool
        @return: item
        @rtype: any type
        '''
        ret = None

        # self.__lock.acquire()
        lock = LockGuard(self.__lock)

        if not uniqId:
            if len(self.__queue):
                uniqId = self.__queue.pop(0)
        if uniqId:
            if erase:
                ret = self.__data.pop(uniqId, None)
            else:
                ret = self.__data.get(uniqId, None)

        # self.__lock.release()

        return ret[0] if ret else None

    def push(self, item, uniqId):
        '''
        @brief: 数据入队列，如果队列已经有了uniqId，插入失败
        @param item: 插入的数据
        @type item: any type
        @return: 插入是否成功
        @rtype: bool
        '''
        begtime = time.time()
        ret = True
        # self.__lock.acquire()
        lock = LockGuard(self.__lock)

        if uniqId in self.__data:
            ret = False
        else:
            self.__data[uniqId] = [item, begtime]
            self.__queue.append(uniqId)
        # self.__lock.release()
        return ret

    def peek(self, uniqId):
        '''
        @brief: 根据uniqId获取item，不会删除item
        @param uniqId: item的id
        @type uniqId: int
        @return: item
        @rtype: any type
        '''
        # self.__lock.acquire()
        lock = LockGuard(self.__lock)

        ret = self.__data.get(uniqId, None)
        # self.__lock.release()
        if not ret:
            return None
        return ret[0]

    def timeout(self):
        '''
        @brief: 检测是否有item超时，如果有就删除
        @return: None
        @rtype: None
        '''
        endtime = time.time()
        # self.__lock.acquire()
        lock = LockGuard(self.__lock)

        # 处理异常情况，防止死锁
        try:
            new_data = {}
            for uniqId, item in self.__data.items():
                if endtime - item[1] < self.__timeout:
                    new_data[uniqId] = item
                else:
                    tarsLogger.debug(
                        'TimeoutQueue:timeout remove id : %d' % uniqId)
            self.__data = new_data
        finally:
            # self.__lock.release()
            pass


class QueueTimeout(threading.Thread):
    """
    超时线程，定时触发超时事件
    """

    def __init__(self, timeout=0.1):
        # threading.Thread.__init__(self)
        tarsLogger.debug('QueueTimeout:__init__')
        super(QueueTimeout, self).__init__()
        self.timeout = timeout
        self.__terminate = False
        self.__handler = None
        self.__lock = threading.Condition()

    def terminate(self):
        tarsLogger.debug('QueueTimeout:terminate')
        self.__terminate = True
        self.__lock.acquire()
        self.__lock.notifyAll()
        self.__lock.release()

    def setHandler(self, handler):
        self.__handler = handler

    def run(self):
        while not self.__terminate:
            try:
                self.__lock.acquire()
                self.__lock.wait(self.timeout)
                self.__lock.release()
                if self.__terminate:
                    break
                self.__handler()
            except Exception as msg:
                tarsLogger.error('QueueTimeout:run exception : %s', msg)

        tarsLogger.debug('QueueTimeout:run finished')


if __name__ == '__main__':
    pass



import json

import stream_gears
if __name__ == '__main__':
    res = stream_gears.get_qrcode()
    print(res)
    res = stream_gears.login_by_qrcode(res)
    with open(f'{json.loads(res)["token_info"]["mid"]}.json', 'w', encoding='utf-8') as file:
        file.write(res)
    print(res)



import aiohttp
from biliup.plugins import random_user_agent

from biliup.common.tars import tarscore
from biliup.plugins import match1
from biliup.plugins.huya_wup.wup_struct import EWebSocketCommandType
from biliup.plugins.huya_wup.wup_struct.WebSocketCommand import HuyaWebSocketCommand
from biliup.plugins.huya_wup.wup_struct.WSUserInfo import HuyaWSUserInfo


class Huya:
    wss_url = 'wss://cdnws.api.huya.com/'
    heartbeat = b'\x00\x03\x1d\x00\x00\x69\x00\x00\x00\x69\x10\x03\x2c\x3c\x4c\x56\x08\x6f\x6e\x6c\x69\x6e\x65\x75' \
                b'\x69\x66\x0f\x4f\x6e\x55\x73\x65\x72\x48\x65\x61\x72\x74\x42\x65\x61\x74\x7d\x00\x00\x3c\x08\x00' \
                b'\x01\x06\x04\x74\x52\x65\x71\x1d\x00\x00\x2f\x0a\x0a\x0c\x16\x00\x26\x00\x36\x07\x61\x64\x72\x5f' \
                b'\x77\x61\x70\x46\x00\x0b\x12\x03\xae\xf0\x0f\x22\x03\xae\xf0\x0f\x3c\x42\x6d\x52\x02\x60\x5c\x60' \
                b'\x01\x7c\x82\x00\x0b\xb0\x1f\x9c\xac\x0b\x8c\x98\x0c\xa8\x0c '
    heartbeatInterval = 60
    # 等待统一ua后修改
    headers = {
        'user-agent': random_user_agent(),
    }

    @staticmethod
    async def get_ws_info(url, context):
        reg_datas = []
        room_id = url.split('huya.com/')[1].split('/')[0].split('?')[0]
        async with aiohttp.ClientSession() as session:
            async with session.get(f'https://www.huya.com/{room_id}', headers=Huya.headers, timeout=5) as resp:
                room_page = await resp.text()
                uid = int(match1(room_page, r"uid\":\"?(\d+)\"?"))
                # tid = match1(room_page, r"lChannelId\":\"?(\d+)\"?")
                # sid = match1(room_page, r"lSubChannelId\":\"?(\d+)\"?")

        import base64
        ws_user_info = HuyaWSUserInfo()
        ws_user_info.iUid = uid
        ws_user_info.bAnonymous = False
        ws_user_info.lGroupId = uid
        ws_user_info.lGroupType = 3
        oos = tarscore.TarsOutputStream()
        ws_user_info.writeTo(oos, ws_user_info)

        # b64data = base64.b64encode(oos.getBuffer())
        # print(b64data)

        ws_cmd = HuyaWebSocketCommand()
        ws_cmd.iCmdType = EWebSocketCommandType.EWSCmd_RegisterReq
        ws_cmd.vData = oos.getBuffer()
        oos = tarscore.TarsOutputStream()
        ws_cmd.writeTo(oos, ws_cmd)

        # oos = tarscore.TarsOutputStream()
        # oos.write(tarscore.int64, 0, uid)
        # oos.write(tarscore.boolean, 1, False)  # Anonymous
        # oos.write(tarscore.string, 2, "")  # sGuid
        # oos.write(tarscore.string, 3, "")
        # oos.write(tarscore.int64, 4, 0)  # tid
        # oos.write(tarscore.int64, 5, 0)  # sid
        # oos.write(tarscore.int64, 6, uid)
        # oos.write(tarscore.int64, 7, 3)

        # b64data = base64.b64encode(oos.getBuffer())
        # print(b64data)

        # wscmd = tarscore.TarsOutputStream()
        # wscmd.write(tarscore.int32, 0, 1)
        # wscmd.write(tarscore.bytes, 1, oos.getBuffer())

        # b64data = base64.b64encode(oos.getBuffer())
        # print(b64data)

        reg_datas.append(oos.getBuffer())

        return Huya.wss_url, reg_datas

    @staticmethod
    def decode_msg(data):
        class User(tarscore.struct):
            @staticmethod
            def readFrom(ios):
                return ios.read(tarscore.string, 2, False).decode("utf8")

        class DColor(tarscore.struct):
            @staticmethod
            def readFrom(ios):
                return ios.read(tarscore.int32, 0, False)

        name = ""
        content = ""
        color = 16777215
        msgs = []
        ios = tarscore.TarsInputStream(data)
        if ios.read(tarscore.int32, 0, False) == 7:
            ios = tarscore.TarsInputStream(ios.read(tarscore.bytes, 1, False))
            if ios.read(tarscore.int64, 1, False) == 1400:
                ios = tarscore.TarsInputStream(ios.read(tarscore.bytes, 2, False))
                name = ios.read(User, 0, False)  # username
                content = ios.read(tarscore.string, 3, False).decode("utf8")  # content
                color = ios.read(DColor, 6, False)  # danmaku color
                if color == -1:
                    color = 16777215
        if name != "":
            msg = {"name": name, "color": f"{color}", "content": content, "msg_type": "danmaku"}
            # else:
            #     msg = {"name": "", "content": "", "msg_type": "other"}
            msgs.append(msg)
        return msgs



from biliup.common.tars import tarscore

class HuyaStreamSettingNotice(tarscore.struct):

    __tars_class__ = "Huya.StreamSettingNotice"

    def __init__(self):
        self.lPresenterUid: tarscore.int64 = 0
        self.iBitRate: tarscore.int32 = 0
        self.iResolution: tarscore.int32 = 0
        self.iFrameRate: tarscore.int32 = 0
        self.lLiveId: tarscore.int64 = 0
        self.sDisplayName: tarscore.string = ""

    @staticmethod
    def writeTo(oos: tarscore.TarsOutputStream, value):
        oos.write(tarscore.int64, 0, value.lPresenterUid)
        oos.write(tarscore.int32, 1, value.iBitRate)
        oos.write(tarscore.int32, 2, value.iResolution)
        oos.write(tarscore.int32, 3, value.iFrameRate)
        oos.write(tarscore.int64, 4, value.lLiveId)
        oos.write(tarscore.string, 5, value.sDisplayName)

    @staticmethod
    def readFrom(ios: tarscore.TarsInputStream):
        value = HuyaStreamSettingNotice()
        value.lPresenterUid = ios.read(tarscore.int64, 0, False)
        value.iBitRate = ios.read(tarscore.int32, 1, False)
        value.iResolution = ios.read(tarscore.int32, 2, False)
        value.iFrameRate = ios.read(tarscore.int32, 3, False)
        value.lLiveId = ios.read(tarscore.int64, 4, False)
        value.sDisplayName = ios.read(tarscore.string, 5, False)
        return value




import asyncio
import json

import httpx
from datetime import datetime, timezone
import logging

try:
    import ssl
    import truststore # type: ignore
except ImportError:
    ssl = None
    truststore = None
    _ssl_context = True
else:
    _ssl_context = truststore.SSLContext(ssl.PROTOCOL_TLS_CLIENT)

DEFAULT_TIMEOUT = httpx.Timeout(
    connect=15.0,
    read=60.0,
    write=60.0,
    pool=15.0,
)
DEFAULT_MAX_RETRIES = 2
DEFAULT_CONNECTION_LIMITS = httpx.Limits(max_connections=100, max_keepalive_connections=100)

client = httpx.AsyncClient(
    http2=True,
    follow_redirects=True,
    timeout=DEFAULT_TIMEOUT,
    limits=DEFAULT_CONNECTION_LIMITS,
    verify=_ssl_context
)
loop = asyncio.get_running_loop()
logger = logging.getLogger('biliup')


def check_timerange(name):
    from biliup.config import config
    
    try:
        time_range_str = config['streamers'].get(name, {}).get('time_range')
        if not time_range_str:
            return True
        time_range = json.loads(time_range_str)
        if not isinstance(time_range, (list, tuple)) or len(time_range) != 2:
            return True

        start = datetime.fromisoformat(time_range[0].replace('Z', '+00:00')).time()
        end   = datetime.fromisoformat(time_range[1].replace('Z', '+00:00')).time()
    except Exception as e:
        logger.error(f'parsing time range {e}')
        return True

    now = datetime.now(timezone.utc).time()

    # Normal interval (e.g. 16:00 → 20:00)
    if start <= end:
        return start <= now <= end

    # Cross‑midnight (e.g. 23:00 → 04:00)
    return now >= start or now <= end



import hashlib
import time
import httpx
import asyncio
from urllib.parse import parse_qs, urlencode, quote
from async_lru import alru_cache
from typing import Union, Any, Optional


from ..common.util import client
from ..config import config
from ..Danmaku import DanmakuClient
from ..engine.decorators import Plugin
from ..engine.download import DownloadBase
from ..plugins import logger, match1, random_user_agent, json_loads, test_jsengine


DOUYU_DEFAULT_DID = "10000000000000000000000000001501"
DOUYU_WEB_DOMAIN = "www.douyu.com"
DOUYU_PLAY_DOMAIN = "playweb.douyucdn.cn"
DOUYU_MOBILE_DOMAIN = "m.douyu.com"


@Plugin.download(regexp=r'https?://(?:(?:www|m)\.)?douyu\.com')
class Douyu(DownloadBase):
    def __init__(self, fname, url, suffix='flv'):
        super().__init__(fname, url, suffix)
        self.room_id: str = ""
        self.douyu_danmaku = config.get('douyu_danmaku', False)
        self.douyu_disable_interactive_game = config.get('douyu_disable_interactive_game', False)
        self.douyu_cdn = config.get('douyu_cdn', 'hw-h5')
        self.douyu_rate = config.get('douyu_rate', 0)
        self.__js_runable = test_jsengine()


    async def acheck_stream(self, is_check=False):
        if len(self.url.split("douyu.com/")) < 2:
            logger.error(f"{self.plugin_msg}: 直播间地址错误")
            return False

        self.fake_headers['referer'] = f"https://{DOUYU_WEB_DOMAIN}"

        try:
            self.room_id = str(match1(self.url, r'rid=(\d+)'))
            if not self.room_id.isdigit():
                self.room_id = await get_real_rid(self.url)
        except:
            logger.exception(f"{self.plugin_msg}: 获取房间号错误")
            return False

        for _ in range(3): # 缓解 #1376 海外请求失败问题
            try:
                room_info = await client.get(
                    f"https://{DOUYU_WEB_DOMAIN}/betard/{self.room_id}",
                    headers=self.fake_headers
                )
                room_info.raise_for_status()
            except httpx.RequestError as e:
                logger.debug(f"{self.plugin_msg}: {e}", exc_info=True)
                continue
            except:
                logger.exception(f"{self.plugin_msg}: 获取直播间信息错误")
                return False
            else:
                break
        else:
            logger.error(f"{self.plugin_msg}: 获取直播间信息失败")
            return False
        room_info = json_loads(room_info.text)['room']

        if room_info['show_status'] != 1:
            logger.debug(f"{self.plugin_msg}: 未开播")
            return False
        if room_info['videoLoop'] != 0:
            logger.debug(f"{self.plugin_msg}: 正在放录播")
            return False
        if self.douyu_disable_interactive_game:
            gift_info = (
                await client.get(
                    f"https://{DOUYU_WEB_DOMAIN}/api/interactive/web/v2/list?rid={self.room_id}",
                    headers=self.fake_headers
            )).json().get('data', {})
            if gift_info:
                logger.debug(f"{self.plugin_msg}: 正在运行互动游戏")
                return False
        self.room_title = room_info['room_name']
        if room_info['isVip'] == 1:
            async with DouyuUtils._lock:
                DouyuUtils.VipRoom.add(self.room_id)

        if is_check:
            return True

        # 提到 self 以供 hack 功能使用
        self.__req_query = {
            'cdn': self.douyu_cdn,
            'rate': str(self.douyu_rate),
            'ver': '219032101',
            'iar': '0', # ispreload? 1: 忽略 rate 参数，使用默认画质
            'ive': '0', # rate? 0~19 时、19~24 时请求数 >=3 为真
            'rid': self.room_id,
            'hevc': '0',
            'fa': '0', # isaudio
            'sov': '0', # use wasm?
        }

        for _ in range(2): # 允许多重试一次以剔除 scdn
            # self.__js_runable = False
            try:
                play_info = await self.aget_web_play_info(self.room_id, self.__req_query)
                if play_info['rtmp_cdn'].startswith('scdn'):
                    new_cdn = play_info['cdnsWithName'][-1]['cdn']
                    logger.debug(f"{self.plugin_msg}: 回避 scdn 为 {new_cdn}")
                    self.__req_query['cdn'] = new_cdn
                    continue
            except (RuntimeError, ValueError) as e:
                logger.warning(f"{self.plugin_msg}: {e}")
            except httpx.RequestError as e:
                logger.debug(f"{self.plugin_msg}: {e}", exc_info=True)
            except Exception as e:
                logger.exception(f"{self.plugin_msg}: 未处理的错误 {e}，自动重试")
            else:
                break
        else:
            # Unknown Error
            logger.error(f"{self.plugin_msg}: 获取播放信息失败")
            return False

        self.raw_stream_url = f"{play_info['rtmp_url']}/{play_info['rtmp_live']}"

        # HACK: 构造 hs-h5 cdn 直播流链接
        # self.douyu_cdn = 'hs-h5'
        if self.douyu_cdn == 'hs-h5' and play_info['rtmp_cdn'] != 'hs-h5':
            if not self.__js_runable:
                logger.warning(f"{self.plugin_msg}: 未找到 jsengine，无法构建 hs-h5 链接")
            is_tct = play_info['rtmp_cdn'] == 'tct-h5'
            try:
                fake_host, cname_url = await self.build_hs_url(self.raw_stream_url, is_tct)
            except:
                logger.exception(f"{self.plugin_msg}: 构建 hs-h5 链接失败")
            else:
                self.raw_stream_url = cname_url
                self.stream_headers['Host'] = fake_host

        return True


    def danmaku_init(self):
        if self.douyu_danmaku:
            content = {
                'room_id': self.room_id,
            }
            self.danmaku = DanmakuClient(self.url, self.gen_download_filename(), content)


    async def aget_sign(self, rid: Union[str, int]) -> dict[str, Any]:
        '''
        :param rid: 房间号
        :return: sign dict
        '''
        if not self.__js_runable:
            raise RuntimeError("jsengine not found")
        try:
            import jsengine
            ctx = jsengine.jsengine()
            js_enc = (
                await client.get(f'https://www.douyu.com/swf_api/homeH5Enc?rids={rid}',
                                 headers=self.fake_headers)
            ).json()['data'][f'room{rid}']
            js_enc = js_enc.replace('return eval', 'return [strc, vdwdae325w_64we];')

            sign_fun, sign_v = ctx.eval(f'{js_enc};ub98484234();') # type: ignore

            tt = str(int(time.time()))
            did = hashlib.md5(tt.encode('utf-8')).hexdigest()
            rb = hashlib.md5(f"{rid}{did}{tt}{sign_v}".encode('utf-8')).hexdigest()
            sign_fun = sign_fun.rstrip(';').replace("CryptoJS.MD5(cb).toString()", f'"{rb}"')
            sign_fun += f'("{rid}","{did}","{tt}");'

            params = parse_qs(ctx.eval(sign_fun))

        except Exception as e:
            logger.exception(f"{self.plugin_msg}: 获取签名参数异常")
            raise e
        return params


    async def aget_web_play_info(
        self,
        room_id: Union[str, int],
        req_query: dict[str, Any],
        req_method: str = "POST",
        did: str = DOUYU_DEFAULT_DID,
    ) -> dict[str, Any]:
        '''
        :param room_id: 房间号
        :param req_query: 请求参数
        :param req_method: 请求方法。可选 GET, POST（默认）
        :param did: douyuid
        :return: PlayInfo
        '''
        if type(room_id) == int:
            room_id = str(room_id)
        if not self.__js_runable:
            s = await DouyuUtils.sign(type="stream", ts=int(time.time()), did=did, rid=room_id)
            logger.debug(f"{self.plugin_msg}: 免 JSEngine 签名参数 {s}")
            auth_param = {
                "enc_data": s['key']['enc_data'],
                "tt": s['ts'],
                "did": did,
                "auth": s['auth'],
            }
            req_query.update(auth_param)
        else:
            s = await self.aget_sign(room_id)
            logger.debug(f"{self.plugin_msg}: JSEngine 签名参数 {s}")
            req_query.update(s)
        api_ver = "V1" if not self.__js_runable else ""
        is_vip = room_id in DouyuUtils.VipRoom # 非 vip room 需要 e 参数，部分直播间可直接请求 hs-h5
        req_method = "GET" if is_vip and not api_ver else "POST"
        path = f"/lapi/live/getH5Play{api_ver}/{room_id}"
        url = f"https://{DOUYU_PLAY_DOMAIN}{path}" if req_method == "GET" else f"https://{DOUYU_WEB_DOMAIN}{path}"
        # url += f"?{urlencode(req_query, doseq=True, encoding='utf-8')}"
        logger.debug(f"{self.plugin_msg}: 使用参数 {str(req_query)} 请求 {url}")
        if req_method == "GET":
            rsp = await client.get(
                url,
                headers=self.fake_headers,
                params=req_query
            )
        else:
            rsp = await client.post(
                url,
                headers={**self.fake_headers, 'user-agent': DouyuUtils.UserAgent},
                params=req_query, # V1 接口需使用查询参数
                data=req_query # 原接口需使用请求体
            )
        rsp.raise_for_status()
        play_data = json_loads(rsp.text)
        if not play_data:
            raise RuntimeError(f"获取播放信息失败 {rsp}")
        if play_data['error'] != 0 or not play_data.get('data', {}):
            raise ValueError(f"获取播放信息错误 {str(play_data)}")
        return play_data['data']


    async def aget_mobile_play_info(
        self,
        req_query: dict[str, Any]
    ) -> dict[str, Any]:
        if not self.__js_runable:
            raise RuntimeError("jsengine not found")
        url = f'https://{DOUYU_MOBILE_DOMAIN}/api/room/ratestream'
        # elif preview:
        #     c_time_str = str(time.time_ns())
        #     url = f'https://playweb.douyucdn.cn/lapi/live/hlsH5Preview/{room_id}?{c_time_str[:18]}'
        #     data = {
        #         'rid': self.__room_id,
        #         'did': data.get('did', ["10000000000000000000000000001501"])[0],
        #     }
        #     req_headers.update({
        #         'Rid': self.__room_id,
        #         'Time': c_time_str[:13],
        #         'Auth': hashlib.md5(f"{self.__room_id}{c_time_str[:13]}".encode('utf-8')).hexdigest(),
        #     })
        rsp = await client.post(
            url,
            headers={**self.fake_headers, 'user-agent': random_user_agent('mobile')},
            data=req_query
        )
        rsp.raise_for_status()
        play_data = json_loads(rsp.text)
        if play_data['code'] != 0:
            raise ValueError(f"获取播放信息错误 {str(play_data)}")
        return play_data['data']


    def parse_stream_info(self, url) -> tuple[str, str, dict]:
        '''
        解析推流信息
        '''
        def get_tx_app_name(rtmp_url) -> str:
            '''
            获取腾讯云推流应用名
            '''
            host = rtmp_url.split('//')[1].split('/')[0]
            app_name = rtmp_url.split('/')[-1]
            # group 按顺序排序
            i = match1(host, r'.+(sa|3a|1a|3|1)')
            if i:
                if i == "sa":
                    i = "1"
                return f"dyliveflv{i}"
            return app_name
        list = url.split('?')
        query = {k: v[0] for k, v in parse_qs(list[1]).items()}
        stream_id = list[0].split('/')[-1].split('.')[0].split('_')[0]
        rtmp_url = list[0].split(stream_id)[0]
        return get_tx_app_name(rtmp_url[:-1]), stream_id, query


    async def build_tx_url(self, tx_app_name, stream_id, query) -> str:
        '''
        构建腾讯CDN URL
        return: tx_url
        '''
        origin = query.get('origin', 'unknown')
        if origin not in ['tct', 'hw', 'dy']:
            '''
            dy: 斗鱼自建
            tct: 腾讯云
            hw: 华为云
            '''
            raise ValueError(f"当前流来源 {origin} 不支持切换为腾讯云推流")
        elif origin == 'dy':
            logger.warning(f"{self.plugin_msg}: 当前流来源 {origin} 可能不存在腾讯云流")
        tx_host = "tc-tct.douyucdn2.cn"
        tx_url = f"https://{tx_host}/{tx_app_name}/{stream_id}.flv?%s"
        m_play_info = await self.aget_mobile_play_info(self.__req_query)
        _, _, m_query = self.parse_stream_info(m_play_info['url'])
        # 需要移动端的宽松验证 token
        m_query.pop('vhost', None)
        query.update({
            'fcdn': 'tct',
            **m_query,
        })
        query = urlencode(query, doseq=True, encoding='utf-8')
        return tx_url % query


    async def build_hs_url(self, url: str, is_tct: bool = False) -> tuple[str, str]:
        '''
        构建火山CDN URL
        :param url: 腾讯云 URL
        :param is_tct: 是否为 tct 流
        return: fake_hs_host, hs_cname_url
        '''
        tx_app_name, stream_id, query = self.parse_stream_info(url)
        # 必须从 tct 转 hs
        if not is_tct:
            url = await self.build_tx_url(tx_app_name, stream_id, query)
        tx_host = url.split('//')[1].split('/')[0]
        hs_host = f"{tx_app_name.replace('dyliveflv', 'huos')}.douyucdn2.cn"
        hs_host = hs_host.replace('huos1.', 'huosa.')
        encoded_url = quote(url, safe='')
        query.update({
            'fp_user_url': encoded_url,
            'vhost': tx_host,
            'domain': tx_host,
        })
        query = urlencode(query, doseq=True, encoding='utf-8')
        hs_cname_host = "douyu-pull.s.volcfcdndvs.com"
        hs_cname_url = f"http://{hs_cname_host}/live/{stream_id}.flv?{query}"
        return (hs_host, hs_cname_url)


class DouyuUtils:
    '''
    逆向实现 //shark2.douyucdn.cn/front-publish/live-master/js/player_first_preload_stream/player_first_preload_stream_6cd7aab.js
    '''
    WhiteEncryptKey: dict = dict()
    VipRoom: set = set()
    # enc_data 会校验 UA
    UserAgent: str = ""
    # 防止并发访问
    _lock: asyncio.Lock = asyncio.Lock()
    _update_key_event: Optional[asyncio.Event] = None

    @staticmethod
    def is_key_valid():
        return (
            bool(DouyuUtils.WhiteEncryptKey) # Key 存在
            and
            DouyuUtils.WhiteEncryptKey.get('expire_at', 0) > int(time.time()) # Key 过期
        )

    @staticmethod
    async def update_key(
        domain: str = DOUYU_WEB_DOMAIN,
        did: str = DOUYU_DEFAULT_DID
    ) -> bool:
        # single-flight
        async with DouyuUtils._lock:
            if DouyuUtils._update_key_event is not None:
                evt = DouyuUtils._update_key_event
                leader = False
            else:
                DouyuUtils._update_key_event = asyncio.Event()
                evt = DouyuUtils._update_key_event
                leader = True
        if not leader:
            await evt.wait()
            return DouyuUtils.is_key_valid()

        try:
            # 防风控
            async with DouyuUtils._lock:
                DouyuUtils.UserAgent = random_user_agent()

            rsp = await client.get(
                f"https://{domain}/wgapi/livenc/liveweb/websec/getEncryption",
                params={"did": did},
                headers={
                    "User-Agent": DouyuUtils.UserAgent
                },
            )
            rsp.raise_for_status()
            data = json_loads(rsp.text)
            if data['error'] != 0:
                raise RuntimeError(f'getEncryption error: code={data["error"]}, msg={data["msg"]}')
            data['data']['cpp']['expire_at'] = int(time.time()) + 86400

            async with DouyuUtils._lock:
                DouyuUtils.WhiteEncryptKey = data['data']
            return True
        except Exception:
            logger.exception(f"{DouyuUtils.__name__}: 获取加密密钥失败")
            return False
        finally:
            async with DouyuUtils._lock:
                if DouyuUtils._update_key_event is not None:
                    DouyuUtils._update_key_event.set()
                    DouyuUtils._update_key_event = None


    @staticmethod
    async def sign(
        type: str, # unused
        ts: int,
        did: str,
        rid: Union[str, int],
    ) -> dict[str, Any]:
        '''
        :param type: unused
        :param ts: 10位Unix时间戳
        :param did: douyuid
        :param rid: 房间号
        '''
        if not rid:
            raise ValueError("rid is None")

        # 确保密钥有效
        for _ in range(2): # 重试两次
            if not DouyuUtils.is_key_valid():
                if not (await DouyuUtils.update_key()):
                    continue
            break
        else:
            raise RuntimeError("获取加密密钥失败")

        if not type:
            type = "stream"
        if not ts:
            ts = int(time.time())
        if not did:
            did = DOUYU_DEFAULT_DID

        rand_str = DouyuUtils.WhiteEncryptKey['rand_str']
        enc_time = DouyuUtils.WhiteEncryptKey['enc_time']
        key = DouyuUtils.WhiteEncryptKey['key']
        is_special = DouyuUtils.WhiteEncryptKey['is_special']
        key_data = {k: v for k, v in DouyuUtils.WhiteEncryptKey.items() if k not in ["cpp"]}

        secret = rand_str
        salt = "" if is_special else f"{rid}{ts}"
        for _ in range(enc_time):
            secret = hashlib.md5(f"{secret}{key}".encode('utf-8')).hexdigest()
        auth = hashlib.md5(f"{secret}{key}{salt}".encode('utf-8')).hexdigest()

        return {
            'key': key_data,
            'alg_ver': "1.0",
            "key_ver": "",
            'auth': auth,
            'ts': ts,
        }


@alru_cache(maxsize=None)
async def get_real_rid(url: str) -> str:
    rid = url.split('douyu.com/')[1].split('/')[0].split('?')[0] or match1(url, r'douyu.com/(\d+)')
    resp = await client.get(f"https://{DOUYU_MOBILE_DOMAIN}/{rid}", headers={
        "User-Agent": random_user_agent('mobile')
    })
    real_rid = match1(resp.text, r'roomInfo":{"rid":(\d+)')
    return str(real_rid)



import random
import string
import json
import requests

import biliup.common.util
from . import logger
from ..engine.decorators import Plugin
from ..engine.download import DownloadBase


@Plugin.download(regexp=r'(?:https?://)?(?:(?:www|m|live)\.)?acfun\.cn')
class Acfun(DownloadBase):
    def __init__(self, fname, url, suffix='flv'):
        super().__init__(fname, url, suffix)

    async def acheck_stream(self, is_check=False):
        if len(self.url.split("acfun.cn/live/")) < 2:
            logger.debug("直播间地址错误")
            return False
        rid = self.url.split("acfun.cn/live/")[1]
        did = "web_"+get_random_name(16)
        cookies = dict(_did=did)
        data1 = {'sid': 'acfun.api.visitor'}
        r1 = await biliup.common.util.client.post("https://id.app.acfun.cn/rest/app/visitor/login",
                                                  headers=self.fake_headers, data=data1, cookies=cookies)
        userid = r1.json()['userId']
        visitorst = r1.json()['acfun.api.visitor_st']
        params = {
            "subBiz": "mainApp",
            "kpn": "ACFUN_APP",
            "kpf": "PC_WEB",
            "userId": str(userid),
            "did": did,
            "acfun.api.visitor_st": visitorst
        }
        data2 = {'authorId': rid, 'pullStreamType': 'FLV'}
        self.fake_headers['referer'] = "https://live.acfun.cn/"
        r2 = await biliup.common.util.client.post("https://api.kuaishouzt.com/rest/zt/live/web/startPlay",
                                                  headers=self.fake_headers, data=data2, params=params)
        if r2.json().get('result') != 1:
            logger.debug(r2.json())
            return False
        d = r2.json()['data']['videoPlayRes']
        self.raw_stream_url = json.loads(d)['liveAdaptiveManifest'][0]['adaptationSet']['representation'][-1]['url']
        self.room_title = r2.json()['data']['caption']
        return True


def get_random_name(numb):
    return random.choice(string.ascii_lowercase) + \
        ''.join(random.sample(string.ascii_letters + string.digits, numb - 1))
